from gettext import gettext as _

import gedit

import gtk
import os.path
import gobject
import sys
import StringIO
import thread
import time
import subprocess

import bzrlib.errors
from bzrlib import workingtree
from bzrlib import branch
from bzrlib import bzrdir
from bzrlib import merge
from bzrlib import globbing
from bzrlib import transport
from bzrlib import log
from bzrlib import check
from bzrlib import osutils
from bzrlib import merge_directive
from bzrlib import config
from bzrlib import reconfigure
from bzrlib import revisionspec

try:
    from bzrlib.plugins import gtk as bzrgtk

except:
    bzrgtk = False


from bzrlib.progress import DummyProgress
from bzrlib.plugin import load_plugins
load_plugins() #right now only used to make "branch lp:bla" work


import dialogs

import widgets

from uifactory import GtkUIFactory
currentdlg = None

ui_str = """<ui>
  <menubar name="MenuBar">
    <menu name="ToolsMenu" action="Tools">
      <placeholder name="BazaarMenu">
        <menu name="BazaarMenu" action="BazaarMenu">

            <menuitem name="BazaarInit" action="BazaarInit"/>
            <separator/>
            <menuitem name="BazaarCommit" action="BazaarCommit"/>

            <menuitem name="BazaarPush" action="BazaarPush"/>
            <menuitem name="BazaarPull" action="BazaarPull"/>
            <menuitem name="BazaarMerge" action="BazaarMerge"/>
            <menuitem name="BazaarUpdate" action="BazaarUpdate"/>
                             
            <menu name="BazaarFile" action="BazaarFile">
                <menuitem name="BazaarAddCurrent" action="BazaarAddCurrent"/>
                <menuitem name="BazaarAddCurrentDir" action="BazaarAddCurrentDir"/>
                <menuitem name="BazaarMVFile" action="BazaarMVFile"/>
                <menuitem name="BazaarIgnoreCurrent" action="BazaarIgnoreCurrent"/>
                <menuitem name="BazaarRemoveCurrent" action="BazaarRemoveCurrent"/>
                <menuitem name="BazaarAnnotate" action="BazaarAnnotate"/>
                <menuitem name="BazaarDiffCurrent" action="BazaarDiffCurrent"/>
                <menuitem name="BazaarLogFile" action="BazaarLogFile"/>
                <menuitem name="BazaarResolveFile" action="BazaarResolveFile"/>
            </menu>
            <menuitem name="BazaarAddAll" action="BazaarAddAll"/>

            <menu name="BazaarBranchInfo" action="BazaarBranchInfo">
                <menuitem name="BazaarStatus" action="BazaarStatus"/>
                <menuitem name="BazaarInfo" action="BazaarInfo"/>
                <menuitem name="BazaarDiffBranch" action="BazaarDiffBranch"/>
                <menu name="BazaarLog" action="BazaarLog">
                    <menuitem name="BazaarLogDir" action="BazaarLogDir"/>
                    <menuitem name="BazaarLogBranch" action="BazaarLogBranch"/>
                </menu>            
                <menuitem name="BazaarViz" action="BazaarViz"/>
                <menuitem name="BazaarConflicts" action="BazaarConflicts"/>
                <menuitem name="BazaarCheck" action="BazaarCheck"/>
                <menuitem name="BazaarMissing" action="BazaarMissing"/>
                <menuitem name="BazaarResolve" action="BazaarResolve"/>
                <menu name="BazaarBranchMaintenance" action="BazaarBranchMaintenance">
                    <menuitem name="BazaarReconcile" action="BazaarReconcile"/>
                    <menuitem name="BazaarUpgrade" action="BazaarUpgrade"/>
                    <menuitem name="BazaarPack" action="BazaarPack"/>
                    <menuitem name="BazaarBreackLock" action="BazaarBreackLock"/>
                    <menuitem name="BazaarReconfigure" action="BazaarReconfigure"/>
                </menu>
                <menuitem name="BazaarTestament" action="BazaarTestament"/>
            </menu>

            <menuitem name="BazaarSettings" action="BazaarSettings"/>
            
            <menu name="BazaarCreatePatch" action="BazaarCreatePatch">
                <menuitem name="BazaarSendPatch" action="BazaarSendPatch"/>
                <menuitem name="BazaarSavePatch" action="BazaarSavePatch"/>
            </menu>
                                    
            <menuitem name="BazaarRevert" action="BazaarRevert"/>
            <menuitem name="BazaarUncommit" action="BazaarUncommit"/>

            <menuitem name="BazaarExport" action="BazaarExport"/>            
            <menuitem name="BazaarPatch" action="BazaarPatch"/>
            <menuitem name="BazaarTags" action="BazaarTags"/>
            <menuitem name="BazaarConfig" action="BazaarConfig"/>
            
        </menu>
      </placeholder>
    </menu>
  </menubar>
  <toolbar name="ToolBar">
    <separator/>
    <toolitem name="BazaarToolButton" action="BazaarToolButton"/>
  </toolbar>
</ui>
"""

class MenuToolAction(gtk.Action):
    __gtype_name__ = "MenuToolAction"

gobject.type_register(MenuToolAction)
MenuToolAction.set_tool_item_type(gtk.MenuToolButton)

class BazaarWindowHelper:
    _statusframe = None
    _statuslbl = None
    _status_img = None
    
    _current_branch = None

    _menu_paths_bzr = ["/MenuBar/ToolsMenu/BazaarMenu/BazaarMenu/BazaarCommit",
                    "/MenuBar/ToolsMenu/BazaarMenu/BazaarMenu/BazaarAddAll",
                    "/MenuBar/ToolsMenu/BazaarMenu/BazaarMenu/BazaarFile/BazaarAddCurrent",
                    "/MenuBar/ToolsMenu/BazaarMenu/BazaarMenu/BazaarFile/BazaarAddCurrentDir",
                    "/MenuBar/ToolsMenu/BazaarMenu/BazaarMenu/BazaarFile/BazaarMVFile",
                    "/MenuBar/ToolsMenu/BazaarMenu/BazaarMenu/BazaarFile/BazaarIgnoreCurrent",
                    "/MenuBar/ToolsMenu/BazaarMenu/BazaarMenu/BazaarFile/BazaarRemoveCurrent",
                    "/MenuBar/ToolsMenu/BazaarMenu/BazaarMenu/BazaarFile/BazaarAnnotate",
                    "/MenuBar/ToolsMenu/BazaarMenu/BazaarMenu/BazaarFile/BazaarDiffCurrent",
                    "/MenuBar/ToolsMenu/BazaarMenu/BazaarMenu/BazaarFile/BazaarLogFile",
                    "/MenuBar/ToolsMenu/BazaarMenu/BazaarMenu/BazaarFile/BazaarResolveFile",
                    "/MenuBar/ToolsMenu/BazaarMenu/BazaarMenu/BazaarPush",
                    "/MenuBar/ToolsMenu/BazaarMenu/BazaarMenu/BazaarPull",
                    "/MenuBar/ToolsMenu/BazaarMenu/BazaarMenu/BazaarMerge",
                    "/MenuBar/ToolsMenu/BazaarMenu/BazaarMenu/BazaarUpdate",
                    "/MenuBar/ToolsMenu/BazaarMenu/BazaarMenu/BazaarBranchInfo/BazaarStatus",
                    "/MenuBar/ToolsMenu/BazaarMenu/BazaarMenu/BazaarBranchInfo/BazaarInfo",
                    "/MenuBar/ToolsMenu/BazaarMenu/BazaarMenu/BazaarBranchInfo/BazaarDiffBranch",
                    "/MenuBar/ToolsMenu/BazaarMenu/BazaarMenu/BazaarBranchInfo/BazaarLog/BazaarLogDir",
                    "/MenuBar/ToolsMenu/BazaarMenu/BazaarMenu/BazaarBranchInfo/BazaarLog/BazaarLogBranch",
                    "/MenuBar/ToolsMenu/BazaarMenu/BazaarMenu/BazaarBranchInfo/BazaarViz",
                    "/MenuBar/ToolsMenu/BazaarMenu/BazaarMenu/BazaarBranchInfo/BazaarConflicts",
                    "/MenuBar/ToolsMenu/BazaarMenu/BazaarMenu/BazaarSettings",
                    "/MenuBar/ToolsMenu/BazaarMenu/BazaarMenu/BazaarCreatePatch/BazaarSendPatch",
                    "/MenuBar/ToolsMenu/BazaarMenu/BazaarMenu/BazaarCreatePatch/BazaarSavePatch",
                    "/MenuBar/ToolsMenu/BazaarMenu/BazaarMenu/BazaarRevert",
                    "/MenuBar/ToolsMenu/BazaarMenu/BazaarMenu/BazaarUncommit",
                    "/MenuBar/ToolsMenu/BazaarMenu/BazaarMenu/BazaarBranchInfo/BazaarCheck",
                    "/MenuBar/ToolsMenu/BazaarMenu/BazaarMenu/BazaarBranchInfo/BazaarMissing",
                    "/MenuBar/ToolsMenu/BazaarMenu/BazaarMenu/BazaarBranchInfo/BazaarBranchMaintenance/BazaarReconcile",
                    "/MenuBar/ToolsMenu/BazaarMenu/BazaarMenu/BazaarBranchInfo/BazaarResolve",
                    "/MenuBar/ToolsMenu/BazaarMenu/BazaarMenu/BazaarBranchInfo/BazaarBranchMaintenance/BazaarUpgrade",
                    "/MenuBar/ToolsMenu/BazaarMenu/BazaarMenu/BazaarBranchInfo/BazaarBranchMaintenance/BazaarPack",
                    "/MenuBar/ToolsMenu/BazaarMenu/BazaarMenu/BazaarBranchInfo/BazaarBranchMaintenance/BazaarBreackLock",
                    "/MenuBar/ToolsMenu/BazaarMenu/BazaarMenu/BazaarBranchInfo/BazaarBranchMaintenance/BazaarReconfigure",
                    "/MenuBar/ToolsMenu/BazaarMenu/BazaarMenu/BazaarBranchInfo/BazaarTestament",
                    "/MenuBar/ToolsMenu/BazaarMenu/BazaarMenu/BazaarExport",
                    "/MenuBar/ToolsMenu/BazaarMenu/BazaarMenu/BazaarPatch",
                    "/MenuBar/ToolsMenu/BazaarMenu/BazaarMenu/BazaarTags",
                    "/MenuBar/ToolsMenu/BazaarMenu/BazaarMenu/BazaarConfig"                 
                    ]
    _menu_paths_nobzr = ["/MenuBar/ToolsMenu/BazaarMenu/BazaarMenu/BazaarInit"]


    def __init__(self, plugin, window):

        self._mypath = os.path.dirname(__file__)
        self._window = window
        self._plugin = plugin
        self._insert_menu()
        self.update_ui()
        self._build_sidebar(window)
        
    # sidebar stuff
    
    def file_click_cb(self,file):
        doc = self._get_doc_from_uri(file)
        if doc == None:
            self._window.create_tab_from_uri("file://"+file,
                                             gedit.encoding_get_current(), 0, False, True)
        else:
            tab = gedit.tab_get_from_document(doc) 
            if tab != None:
                self._window.set_active_tab(tab)       
        
    def _build_sidebar(self,window):
        panel = self._window.get_side_panel()

        
        self._treeview = widgets.FileListTreeView(fileclickcb = self.file_click_cb)
        #self._filter = widgets.FilterBar(window)
        scroll = gtk.ScrolledWindow()
        scroll.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
        scroll.add(self._treeview)

        widget = gtk.VBox()
        #widget.pack_start(self._filter, False, False)
        
        
        widget.pack_start(scroll)
        widget.show_all()

        image = gtk.Image()
        image.set_from_file(self._mypath+"/img/bzr-icon-16.png") #it's our lovely icon :)
        panel.add_item(widget, "Bazaar", image)
        
        map(self._register_document, gedit.app_get_default().get_documents())
        self._window.connect("active-tab-changed", self._active_tab_changed)
        self._window.connect("tab-added", self._tab_added)
        
        self._treeview.connect("button_press_event", self._sidebar_click)
        self._build_sidebar_menu()
        
        
    
    def _sidebar_click(self, menu, event):
        if event.button == 3:
            self._sb_menu['menu'].popup(None, None, None, 0,
                                            event.time)
    
    def _get_selected_file(self):
        sel = self._treeview.get_selected()
        if sel != None:
            return self._treeview.path + sel
        else:
            return None
            
    def _on_sb_menu_open(self, menu):
        file = self._get_selected_file()
        if file != None:
            if not os.path.isdir(fullpath):
                self.file_click_cb(fullpath)
            else:
                self._treeview.set_path(fullpath)
                
    def _on_sb_menu_resolve(self, menu):
        file = self._get_selected_file()
        self._bzr_resolve(file, 'file')
        self._treeview.draw_tree()
        
    def _on_sb_menu_add(self, menu):
        file = self._get_selected_file()
        self._bzr_add(file)
        self._treeview.draw_tree()
    
    def _on_sb_menu_ignore(self, menu):
        file = self._get_selected_file()
        self._bzr_ignore(file)
        self._treeview.draw_tree()
        
    def _on_sb_menu_remove(self, menu):
        file = self._get_selected_file()
        self._bzr_remove(file)
        self._treeview.draw_tree()
    
    def _on_sb_menu_annotate(self, menu):
        file = self._get_selected_file()
        self._bzr_annotate(file)
        
    def _on_sb_menu_diff(self, menu):
        file = self._get_selected_file()
        self._bzr_diff_file(file)
        
    def _on_sb_menu_log(self, menu):
        file = self._get_selected_file()
        self._bzr_log(file,'file')        
    
    def _on_sb_menu_refresh(self, menu):
        self._treeview.draw_tree()
               
                    
    def _build_sidebar_menu(self):
        self._sb_menu = {}
        
        self._sb_menu['menu'] = gtk.Menu()
        self._sb_menu['open'] = gtk.ImageMenuItem(gtk.STOCK_OPEN)
        self._sb_menu['open'].connect("activate",self._on_sb_menu_open)


        self._sb_menu['refresh'] = gtk.ImageMenuItem("Refresh")
        self._sb_menu['refresh'].connect("activate",self._on_sb_menu_refresh)
        img = gtk.Image()
        img.set_from_file(self._mypath+"/img/icons/refresh16.png")
        self._sb_menu['refresh'].set_image(img)
                
        self._sb_menu['add'] = gtk.MenuItem("Add to bazaar")
        self._sb_menu['add'].connect("activate",self._on_sb_menu_add)
        
        self._sb_menu['ignore'] = gtk.MenuItem("Ignore")
        self._sb_menu['ignore'].connect("activate",self._on_sb_menu_ignore)
        
        self._sb_menu['remove'] = gtk.MenuItem("Remove from bazaar")
        self._sb_menu['remove'].connect("activate",self._on_sb_menu_remove)
        
        self._sb_menu['annotate'] = gtk.MenuItem("Annotate")
        self._sb_menu['annotate'].connect("activate",self._on_sb_menu_annotate)        
        
        self._sb_menu['diff'] = gtk.ImageMenuItem("View diff")
        self._sb_menu['diff'].connect("activate",self._on_sb_menu_diff)
        img = gtk.Image()
        img.set_from_file(self._mypath+"/img/icons/diff16.png")
        self._sb_menu['diff'].set_image(img)
        
        self._sb_menu['log'] = gtk.ImageMenuItem("View Log")
        self._sb_menu['log'].connect("activate",self._on_sb_menu_log)
        img = gtk.Image()
        img.set_from_file(self._mypath+"/img/icons/log16.png")
        self._sb_menu['log'].set_image(img)
        
        self._sb_menu['resolve'] = gtk.MenuItem("Mark resolved")
        self._sb_menu['resolve'].connect("activate",self._on_sb_menu_resolve)
        

        
        self._sb_menu['menu'].append(self._sb_menu['open'])
        self._sb_menu['menu'].append(self._sb_menu['refresh'])
        self._sb_menu['menu'].append(self._sb_menu['add'])
        self._sb_menu['menu'].append(self._sb_menu['ignore'])
        self._sb_menu['menu'].append(self._sb_menu['remove'])
        self._sb_menu['menu'].append(self._sb_menu['annotate'])
        self._sb_menu['menu'].append(self._sb_menu['diff'])
        self._sb_menu['menu'].append(self._sb_menu['log'])
        self._sb_menu['menu'].append(self._sb_menu['resolve'])
        
        self._sb_menu['menu'].show_all()
        
        
    def _register_document(self, doc):
        if doc.get_data("gedit-bzr-saved-signal-id"):
            return
        id = doc.connect("saved", self._document_saved)
        doc.set_data("gedit-bzr-saved-signal-id", id)
        
    def _tab_added(self, window, tab):
        self._register_document(tab.get_document())
        
    def _document_saved(self, document, arg1):
        self._treeview.update_file(document.get_uri_for_display())
        
    def _active_tab_changed(self, window, tab):
        gobject.idle_add(self._do_set_active_uri_from_tab, tab)  
        
    def _do_set_active_uri_from_tab(self, tab):
        update = False
        dir = self._get_active_doc_dir()
        if self._get_is_active_branch(dir):
            curbranch = branch.Branch.open_containing(dir)[0]        
        else:
            curbranch = None
        
        if self._current_branch == None or (curbranch != None and self._current_branch._base != curbranch._base):
            currpath = self._get_active_doc_dir()
            if currpath != None: # FIXME: should only update if branch is different between old and new file
                self._treeview.set_path(currpath)
            self._current_branch = curbranch
        elif curbranch == None:
            self._treeview.clear_tree()

    # bzr functions
    def _bzr_log(self, file, log_type):
        try:
            
            if log_type != 'dir':
                br ,relpath = branch.Branch.open_containing(file)
            else:
                br ,relpath = branch.Branch.open_containing(self._get_active_doc_dir())
            to_file = StringIO.StringIO()
            lf = log.LongLogFormatter(to_file=to_file)
            if log_type == 'branch':
                log.show_log(br, lf)
                # FIXME: this should have a "find" function...
            else:
                tree = br.bzrdir.open_workingtree()
                file_id = tree.path2id(relpath)
                if file_id == 'TREE_ROOT':
                    log.show_log(br, lf) # if it's tree root let's show branch
                else:
                    log.show_log(br, lf,file_id)
                    
            dialogs._bzrInfoDialog("Branch Log",to_file.getvalue(),[400,250])
            
        except:
            dialogs._bzrErrorDialog("Bazaar error showing log." ,sys.exc_value)
    
    def _bzr_resolve(self, file, log_type):
        try:
            from bzrlib.conflicts import resolve
            wt,relpath = workingtree.WorkingTree.open_containing(file)
            
            if log_type == 'file':
                resolve(wt, [relpath])
            else:
                resolve(wt,recursive=True)
        except:
            dialogs._bzrErrorDialog("Bazaar error resolving." ,sys.exc_value)        
        
    def _bzr_diff_file(self, file):
        try:
            new_tree ,relpath = workingtree.WorkingTree.open_containing(file)
            old_revid = new_tree.branch.revision_history()[-2]
            old_tree = new_tree.branch.repository.revision_tree(old_revid)
            
            from bzrlib.diff import _get_trees_to_diff, show_diff_trees
            
            to_file = StringIO.StringIO()
            
            if type == 'branch':
                relpath = ''
            
            show_diff_trees(old_tree, new_tree, to_file,
                            specific_files=[relpath,])
            
            dialogs._bzrDiffDialog("File diff: %s" % relpath,to_file.getvalue())
        except:
            dialogs._bzrErrorDialog("Bazaar error showing diff." ,sys.exc_value)   
    
    def _bzr_annotate(self, file):
        if self._check_bzrgtk("annotate"):
            try:
                from bzrlib.plugins.gtk.annotate.gannotate import GAnnotateWindow
                from bzrlib.plugins.gtk.annotate.config import GAnnotateConfig
                br ,relpath = branch.Branch.open_containing(file)
                
                tree = br.basis_tree()
                
                file_id = tree.path2id(relpath)
                if file_id != None:                   
                    if bzrlib.plugins.gtk.version_info[1] < 94:
                        window = GAnnotateWindow(False, False)
                    else:
                        window = GAnnotateWindow(False, False, branch=br)
                    window.connect("destroy", lambda w: window.destroy())
                    config = GAnnotateConfig(window)
                    window.annotate(tree, br, file_id)
                    window.show()
                else:
                    raise Exception("File not in branch")
                
            except:
                dialogs._bzrErrorDialog("Bazaar error showing annotate window." ,sys.exc_value) 
    
    def _bzr_remove(self, file):
        try:
            wt, name_pattern = workingtree.WorkingTree.open_containing(file)
            wt.remove(file)
            self._statusbar_draw("File %s removed" % file)    
        except:
            dialogs._bzrErrorDialog("Bazaar error removing %s." % file,sys.exc_value)
            
    
    def _bzr_ignore(self, file):
        wt, name_pattern = workingtree.WorkingTree.open_containing(file)
        if wt.is_ignored(file) is None:
            from bzrlib.atomicfile import AtomicFile
            
            ifn = wt.abspath('.bzrignore')
            if os.path.exists(ifn):
                f = open(ifn, 'rt')
                try:
                    igns = f.read().decode('utf-8')
                finally:
                    f.close()
            else:
                igns = ''
            if igns and igns[-1] != '\n':
                igns += '\n'
            
            igns += name_pattern + '\n'
            
            f = AtomicFile(ifn, 'wb')
            try:
                f.write(igns.encode('utf-8'))
                f.commit()
            finally:
                f.close()
    
            if not wt.path2id('.bzrignore'):
                wt.add(['.bzrignore'])
                
            ignored = globbing.Globster([name_pattern])
            matches = []
            wt.lock_read()
            
            for entry in wt.list_files():
                id = entry[3]
                if id is not None:
                    filename = entry[0]
                    if ignored.match(filename):
                        matches.append(filename.encode('utf-8'))
            wt.unlock()
            if len(matches) > 0:
                dialogs._bzrErrorDialog("The following files are version controlled and" \
                      " match your ignore pattern:\n%s" % ("\n".join(matches),),msgtype='Warning')
            self._statusbar_draw("File added to ignore list")
        else:
            dialogs._bzrErrorDialog("This file is already in the ignore list")
    
    def _bzr_add(self, file, recursive=False):
        try:
            wt,relpath = workingtree.WorkingTree.open_containing(file)
        except:
            dialogs._bzrErrorDialog("Bazaar error opening %s to add." % file,sys.exc_value)
        else:
            try:
                ret = wt.smart_add([file],recursive)
            except:
                dialogs._bzrErrorDialog("Bazaar error adding file %s" % file,sys.exc_value)
            else:
                self._statusbar_draw("File %s added" % file)
    

    # UI Callbacks
    
    def newworkspace_cb(self, data, dialog):
        
        location,isrepo,createaction,branchname,parentlocation = data

        if isrepo:
            try:
                dialog.set_status("Creating shared repository...")
                format = bzrdir.format_registry.make_bzrdir('default')
                to_transport = transport.get_transport(location)
                to_transport.ensure_base()
        
                newdir = format.initialize_on_transport(to_transport)
                repo = newdir.create_repository(shared=True)
                repo.set_make_working_trees(True)
            except:
                dialogs._bzrErrorDialog("Bazaar creating repository.",sys.exc_value)
            else:
                if createaction != 'init':
                    location = os.path.join(location,branchname)


        if createaction == 'init' and not isrepo: #create empty branch
            try:
                dialog.set_status("Creating local branch")
                wt = bzrlib.bzrdir.BzrDir.create_standalone_workingtree(location)
            except:
                dialogs._bzrErrorDialog("Bazaar error executing Init",sys.exc_value)
            else:
                self.update_ui()
                self._statusbar_draw("Directory %s innited" % location)
        elif createaction == 'init-branch':
            try:
                dialog.set_status("Branching from remote location...")
                to_transport =  transport.get_transport(location)
                create_branch = bzrdir.BzrDir.create_branch_convenience
                create_branch(to_transport.base, format=None,
                                       possible_transports=[to_transport])
            except:
                dialogs._bzrErrorDialog("Bazaar error creating branch in shared repository",sys.exc_value)
            else:
                self.update_ui()
                self._statusbar_draw("Directory %s branched" % location)
        elif createaction == 'branch':
            try:
                dialog.set_status("Opening remote branch...")
                onlinebranch = branch.Branch.open(parentlocation)
                dialog.set_status("Creating local branch...")
                localbranch = onlinebranch.bzrdir.sprout(location)
            except:
                dialogs._bzrErrorDialog("Bazaar error branching",sys.exc_value)
            else:
                self.update_ui()
                self._statusbar_draw("Branched %s to %s" % (parentlocation, location))
        elif createaction == 'checkout':
            try:
                dialog.set_status("Opening remote location...")
                acel_trans, onlinebranch = bzrdir.BzrDir.open_tree_or_branch(parentlocation)
                dialog.set_status("Creating local checkout...")
                onlinebranch.create_checkout(location,None,False,acel_trans,False)
            except :
                dialogs._bzrErrorDialog("Bazaar error doing checkout",sys.exc_value)
            else:
                self._statusbar_draw("Checked out %s to %s" % (parentlocation, location))
                        
    def on_bazaar_init(self, menu):
        pid = subprocess.Popen(['%s/commands.py' % self._mypath, 'c="newworkspace"']).pid
    
    def on_bazaar_commit(self, menu):
        docuri = self._get_active_doc_uri_exists()
        if docuri is None:
            dialogs._bzrErrorDialog("This action must be done from a saved document.")
        else:
            try:
                wt,relpath = workingtree.WorkingTree.open_containing(docuri)
            except:
                dialogs._bzrErrorDialog("Bazaar error doing commit",sys.exc_value)
            else:
                    
                dlgoptions = {"--author":["Author",'T'],
                           "--local": ["Local commit","B"],
                           "--unchanged":["Commit even unchanged","B"],
                           "--strict": ["Strict","B"]}
                
                commitdlg = dialogs.BzrComitDialog(wt.basedir,dlgoptions,wt)
                response, comment, retoptions, files = commitdlg.run()
                files_list = files[0]
                files_info = files[1]
                
                commitdlg.disable_widgets()
                if response == gtk.RESPONSE_OK:
                    try:
                        commitdlg.set_status("Committing...")
                        author = retoptions['--author']
                        local = retoptions['--local']
                        unchanged = retoptions['--unchanged']
                        strict = retoptions['--strict']
                
                        if wt.branch.repository.is_locked():
                            wt.branch.repository.unlock()
                
                        wt.commit(comment,author=author,strict=strict,local=local,allow_pointless=unchanged,
                                        specific_files=files_list)
            
                    except:
                        dialogs._bzrErrorDialog("Bazaar error doing commit",sys.exc_value)
                    else:
                        self._statusbar_draw("Commit on  %s ok" % wt.basedir) # FIXME: should say also revision num?
                commitdlg.destroy()


    def on_bazaar_add(self, menu):
        localpath = self._get_active_doc_dir()
        if localpath is None:
            localpath = ''

        if localpath == '':
            dialogs._bzrErrorDialog("This action must be done from a saved document.")
        else:
            try:
                wt,relpath = workingtree.WorkingTree.open_containing(localpath)
            except:
                dialogs._bzrErrorDialog("Bazaar error opening %s to add." % localpath,sys.exc_value)
            else:
                
                self._bzr_add(wt.basedir)

    def on_bazaar_add_current(self,menu):
        docuri = self._get_active_doc_uri_exists()
        if docuri is None:
            dialogs._bzrErrorDialog("This action must be done from a saved document.")
        else:
            self._bzr_add_file(docuri)
    
    def on_bazaar_add_current_dir(self,menu):
        localpath = self._get_active_doc_dir()
        if docuri is None:
            dialogs._bzrErrorDialog("This action must be done from a saved document.")
        else:
            self._bzr_add(localpath,True)
                    
    def on_bazaar_push(self, menu):      
        localpath = self._get_active_doc_dir()
        if localpath is None:
            dialogs._bzrErrorDialog("This action must be done from a saved document.")
        else:
            pid = subprocess.Popen(['%s/commands.py' % self._mypath, 'c="push"', 'l="%s"' % localpath]).pid


    def on_bazaar_pull(self, menu):

        localpath = self._get_active_doc_dir()
        if localpath == None:
            dialogs._bzrErrorDialog("This action must be done from a saved document.")
        else:
            pid = subprocess.Popen(['%s/commands.py' % self._mypath, 'c="pull"', 'l="%s"' % localpath]).pid

    def on_bazaar_merge(self, menu):
        
        localpath = self._get_active_doc_dir()
        if localpath == None:
            dialogs._bzrErrorDialog("This action must be done from a saved document.")
        else:
            pid = subprocess.Popen(['%s/commands.py' % self._mypath, 'c="merge"', 'l="%s"' % localpath]).pid

    
    def on_bazaar_update(self, menu):
        localpath = self._get_active_doc_dir()
        if localpath == None:
            dialogs._bzrErrorDialog("This action must be done from a saved document.")
        else:
            try:
                wt,relpath = workingtree.WorkingTree.open_containing(localpath)
            except:
                dialogs._bzrErrorDialog("Bazaar error updating." ,sys.exc_value)
            else:
                wt.update()
                self._statusbar_draw("Updated %s" % localpath)

    def on_bazaar_status(self,menu):
        localpath = self._get_active_doc_dir()
        if localpath == None:
            dialogs._bzrErrorDialog("This action must be done from a saved document.")
        else:
            try:
                wt ,relpath = workingtree.WorkingTree.open_containing(localpath)
                old_tree = wt.branch.repository.revision_tree(wt.branch.last_revision())
                delta = wt.changes_from(old_tree)
                if delta.has_changed():
                    dialogs._bzrInfoDialog("Status",delta.get_changes_as_text(),[250,300])
                else:
                    dialogs._bzrInfoDialog("Status", "This branch has no changes")
            except:
                dialogs._bzrErrorDialog("Bazaar error showing status." ,sys.exc_value)            
    
    def on_bazaar_info(self,menu):
        localpath = self._get_active_doc_dir()
        if localpath == None:
            dialogs._bzrErrorDialog("This action must be done from a saved document.")
        else:
            try:
                bd ,relpath = bzrdir.BzrDir.open_containing(localpath)
                noise_level = 2
                outfile = StringIO.StringIO()
                from bzrlib.info import show_bzrdir_info
                show_bzrdir_info(bd,
                                 verbose=noise_level, outfile=outfile)
        
                dialogs._bzrInfoDialog("Info",outfile.getvalue(),[400,300])

            except:
                dialogs._bzrErrorDialog("Bazaar error showing info." ,sys.exc_value)
        
    def on_bazaar_annotate(self,menu):
        if self._check_bzrgtk("annotate"):
            docuri = self._get_active_doc_uri_exists()
            if docuri == None:
                dialogs._bzrErrorDialog("This action must be done from a saved document.")
            else:
                self._bzr_annotate(docuri)           


    
    def on_bazaar_branchsettings(self,menu):
        
        localpath = self._get_active_doc_dir()
        if localpath == None:
            dialogs._bzrErrorDialog("This action must be done from a saved document.")
        else:
            try:
                br, relpath = branch.Branch.open_containing(localpath)
                branchsettingsdlg = dialogs.BzrBranchSettings(br)
                branchsettingsdlg.show_all()
            except:
                dialogs._bzrErrorDialog("Bazaar error launching settings dialog." ,sys.exc_value)            
               
    def on_bazaar_send(self,menu,action):
        localpath = self._get_active_doc_dir()
        if localpath == None:
            dialogs._bzrErrorDialog("This action must be done from a saved document.")
        else:
            inputdlg = None
            try:
                br, relpath = branch.Branch.open_containing(localpath)
                submitloc = br.get_submit_branch()
                if submitloc == None:
                    submitloc = br.get_parent()
                
                config = br.get_config()

                dlgoptions = {"--remember":["Remember submit location",'B'],
                           "--message":["Message","T"],
                           "--revision":["Revision","R",br],
                           "--no-patch":["Do not include a preview patch","B"],
                           "--no-bundle": ["Do not include a bundle","B"]}
                                    
                                    

                
                if action == 'save':
                    inputdlg = dialogs.BzrDobleInputFileDialog(submitloc,
                             "" ,"Send your patch", "Enter submit branch location","Save patch fo file" ,"Send patch",dlgoptions)
                    ret, newsubmitloc, patchfile, retpotions = inputdlg.run()
                else:
                    mailto = config.get_user_option('submit_to')
                    if mailto == None:
                        mailto = ''
                    inputdlg = dialogs.BzrDobleInputDialog(submitloc, mailto ,"Send your patch", "Enter submit branch location","Enter destination email addres" ,"Send patch",dlgoptions)

                    ret, newsubmitloc, mailto, retpotions = inputdlg.run()
                    inputdlg.disable_widgets()
                    
                opt_remember = retpotions['--remember']
                opt_message = retpotions['--message']
                opt_revision = retpotions['--revision']
                opt_nopatch = retpotions['--no-patch']
                opt_nobundle = retpotions['--no-bundle']
                
                
                                
                if ret == gtk.RESPONSE_OK:
                    if newsubmitloc != '':
                        if opt_remember:
                            opt_remember = newsubmitloc
                        else:  
                            opt_remember = None
                        
                        inputdlg.set_status("Generating...")
                        revision_id = br.last_revision() # FIXME: dialog should allow to select revision?
                        base_revision_id = None
                        directive = merge_directive.MergeDirective2.from_objects(
                            br.repository, opt_revision, time.time(),
                            osutils.local_time_offset(), newsubmitloc,
                            public_branch=opt_remember, include_patch=not opt_nopatch,
                            include_bundle=not opt_nobundle, opt_message=None,
                            base_revision_id=opt_revision)
                        
                        
                        if action == 'send':
                            if mailto != '':
                                inputdlg.set_status("Opening mail client...")
                                sendfile = StringIO.StringIO()
                                sendfile.writelines(directive.to_lines())
                                mail_client = config.get_mail_client()
                                basename = directive.get_disk_name(br)
                                mail_client.compose_merge_request(mailto, "[MERGE]",
                                                          sendfile.getvalue(), basename)
                            else:
                                dialogs._bzrErrorDialog("You need to enter a destination email address")
                        else:
                            inputdlg.set_status("Saving file...")
                            outfile = open(patchfile, 'wb')
                            outfile.writelines(directive.to_lines())
                            outfile.close()
                        
                    else:
                        dialogs._bzrErrorDialog("You need to enter a file to save your patch")
                #
                
            except:
                dialogs._bzrErrorDialog("Bazaar error showing status." ,sys.exc_value)      
                
            if inputdlg != None:
                inputdlg.destroy()
            
    def on_bazaar_move(self, menu):
        docuri = self._get_active_doc_uri_exists()
        if docuri is None:
            dialogs._bzrErrorDialog("This action must be done from a saved document.")
        else:
            try:
                wt,relpath = workingtree.WorkingTree.open_containing(docuri)
                
    
                inputdlg = dialogs.BzrInputDialog(wt.basedir,"Move file ./%s to:" % relpath, "Select where you want to move your file","Move file")
                retcode, newpath = inputdlg.run()
                
                names_list = [docuri,newpath]
                rel_names = [wt.relpath(docuri),wt.relpath(newpath)]
                after = False
    
                into_existing = osutils.isdir(names_list[-1])
                tree = wt
    
                wt.lock_write()
                
                if into_existing and len(names_list) == 2:
                    inv = tree.inventory
                    from_id = tree.path2id(rel_names[0])
                    if (not osutils.lexists(names_list[0]) and
                        from_id and inv.get_file_kind(from_id) == "directory"):
                        into_existing = False
                # move/rename
                if into_existing:
                    # move into existing directory
                    for pair in tree.move(rel_names[:-1], rel_names[-1], after=after):
                        #print("%s => %s\n" % pair)
                        pass
                else:
                    if len(names_list) != 2:
                        raise errors.BzrCommandError('to mv multiple files the'
                                                     ' destination must be a versioned'
                                                     ' directory')
                    tree.rename_one(rel_names[0], rel_names[1], after=after)
                    #print ("%s => %s\n" % (rel_names[0], rel_names[1]))
                wt.unlock()
                ###
            except:
                wt.unlock()
                dialogs._bzrErrorDialog("Bazaar error opening %s to add." % docuri,sys.exc_value)
            inputdlg.destroy()

    def on_bazaar_diff(self,menu,type):
        docuri = self._get_active_doc_uri_exists()
        if docuri is None:
            dialogs._bzrErrorDialog("This action must be done from a saved document.")
        else:
            self._bzr_diff_file(docuri)         
            
    def on_bazaar_log(self,action,log_type):

        docuri = self._get_active_doc_uri_exists()
        if docuri is None:
            dialogs._bzrErrorDialog("This action must be done from a saved document.")
        else:
            self._bzr_log(docuri, log_type)


    
    def on_bazaar_resolve(self,aciton,log_type):
        docuri = self._get_active_doc_uri_exists()
        if docuri is None:
            dialogs._bzrErrorDialog("This action must be done from a saved document.")
        else:
            self._bzr_resolve(docuri, log_type)
            self._treeview.draw_tree()
        
    def on_bazaar_revert(self,menu):
        localpath = self._get_active_doc_dir()
        if localpath == None:
            dialogs._bzrErrorDialog("This action must be done from a saved document.")
        else:
            try:
                wt,relpath = workingtree.WorkingTree.open_containing(localpath)
            except:
                dialogs._bzrErrorDialog("Bazaar error doing revert." ,sys.exc_value)
            else:
                dlgoptions = {"--forget-merges":["Remove pending merge marker",'B'],
                           "--no-backup":["Do not save backups of reverted files.","B"],
                           "--revision":["Revision","R",wt.branch]}
                
                confirmdlg = dialogs.BzrConfirmDialog("Revert", "Confirm revert of changes",options=dlgoptions)
                resp, ret_options = confirmdlg.run()
                
                forgetmerges = ret_options['--forget-merges']
                revisionid = ret_options['--revision']
                nobackup = ret_options['--no-backup']
                
                try:
                    if resp == gtk.RESPONSE_OK:
                        if forgetmerges:
                            wt.set_parent_ids(wt.get_parent_ids()[:1])
                        else:
                            if revisionid == '':
                                revisionid = wt.branch.last_revision()
                            
                            ret = wt.revert(old_tree=wt.branch.repository.revision_tree(revisionid),
                                            backups=not nobackup)

                except:
                    dialogs._bzrErrorDialog("Bazaar error doing revert." ,sys.exc_value)
                else:
                    self._statusbar_draw("Reverted all files in %s" % localpath)
                confirmdlg.destroy()

    def on_bazaar_uncommit(self,menu):
        localpath = self._get_active_doc_dir()
        if localpath == None:
            dialogs._bzrErrorDialog("This action must be done from a saved document.")
        else:
            confirmdlg = None
            try:
                control, relpath = bzrdir.BzrDir.open_containing(localpath)
                tree = control.open_workingtree()
                br = tree.branch
                                
                dlgoptions = {
                           "--local":["Only remove the local commits.","B"],
                           "--revision":["Revision","R",br]}
                
                confirmdlg = dialogs.BzrConfirmDialog("UnCommit", "Confirm uncommit",options=dlgoptions)
                resp, ret_options = confirmdlg.run()
                
                
                opt_local = ret_options['--local']
                opt_revision = ret_options['--revision']
                
                from bzrlib.log import log_formatter, show_log
                from bzrlib.uncommit import uncommit
        
                last_revno, last_rev_id = br.last_revision_info()
                
                rev_id = None
                if opt_revision == '':
                    revno = last_revno
                    rev_id = last_rev_id
                    revision = revisionspec.RevisionSpec.from_string(opt_revision)
                else:
                    revision = revisionspec.RevisionSpec.from_string("revid:%s" % opt_revision)
                    
                revno = revision.in_history(br).revno + 1
                
                if revno <= last_revno:
                    rev_id = br.get_rev_id(revno)
                    
                # something is locked and i can't find what...
                if br.repository.is_locked():
                    br.repository.unlock()
                try:
                    br.unlock()
                except:
                    pass
                try:
                    tree.unlock()
                except:
                    pass
                
                uncommit(br, tree=tree, revno=revno, local=opt_local)
                                
            except:
                dialogs._bzrErrorDialog("Bazaar error doing uncommit." ,sys.exc_value)
            else:
                self._statusbar_draw("Uncommited las commit in %s" % localpath)
            if confirmdlg != None:
                confirmdlg.destroy()

                            
    def on_bazaar_ignorecurrent(self,menu):
        docuri = self._get_active_doc_uri_exists()
        if docuri:
            response = dialogs._bzrQuestionDialog("Ignore","Make bazaar ignore <b>%s</b>?" % docuri,"Ignore file")
            if response == gtk.RESPONSE_YES:
                self._bzr_ignore(docuri)
        else:
            dialogs._bzrErrorDialog("This action must be done from a saved document.")
            
            
    def on_bazaar_removecurrent(self,menu):
        docuri = self._get_active_doc_uri_exists()
        if docuri:
            response = dialogs._bzrQuestionDialog("Remove","Make bazaar remove <b>%s</b>?" % docuri,"Remove file")
            if response == gtk.RESPONSE_YES:
                self._bzr_remove(docuri)
        else:
            dialogs._bzrErrorDialog("This action must be done from a saved document.")
            
    def on_bazaar_viz(self, menu):
        if self._check_bzrgtk("visualize"):
            localpath = self._get_active_doc_dir()
            if localpath == None:
                dialogs._bzrErrorDialog("This action must be done from a saved document.")
            else:
                try:
                    br, relpath = branch.Branch.open_containing(localpath)
                    rv = [br.last_revision(),] # FIXME: we should allow choosing a revision
                    vizwin = bzrgtk.start_viz_window(br,rv,None)
                    vizwin.show()
                except:
                    dialogs._bzrErrorDialog("Bazaar error loading visualize plugin.", sys.exc_value)

    def on_bazaar_check(self,menu):
        localpath = self._get_active_doc_dir()
        if localpath == None:
            dialogs._bzrErrorDialog("This action must be done from a saved document.")
        else:
            try:
                br, relpath = branch.Branch.open_containing(localpath)
                checkobj = check.Check(br.repository)
                checkobj.check()
                
                checkreport = "Checked %s" % localpath
                checkreport = checkreport + "\n" + ('%6d revisions' % checkobj.checked_rev_cnt)
                checkreport = checkreport + "\n" + ('%6d file-ids' % len(checkobj.checked_weaves))
                checkreport = checkreport + "\n" + ('%6d unique file texts' % checkobj.checked_text_cnt)
                checkreport = checkreport + "\n" + ('%6d repeated file texts' % checkobj.repeated_text_cnt)
                checkreport = checkreport + "\n" + ('%6d unreferenced text versions' %
                     len(checkobj.unreferenced_versions))
                if checkobj.missing_inventory_sha_cnt:
                    checkreport = checkreport + "\n" + ('%6d revisions are missing inventory_sha1' %
                         checkobj.missing_inventory_sha_cnt)
                if checkobj.missing_revision_cnt:
                    checkreport = checkreport + "\n" + ('%6d revisions are mentioned but not present' %
                         checkobj.missing_revision_cnt)
                if len(checkobj.ghosts):
                    checkreport = checkreport + "\n" + ('%6d ghost revisions' % len(checkobj.ghosts))

                if len(checkobj.missing_parent_links):
                    checkreport = checkreport + "\n" + ('%6d revisions missing parents in ancestry' %
                         len(checkobj.missing_parent_links))

                if len(checkobj.inconsistent_parents):
                    checkreport = checkreport + "\n" + ('%6d inconsistent parents' % len(checkobj.inconsistent_parents))

                if checkobj.revs_with_bad_parents_in_index:
                    checkreport = checkreport + "\n" + ('%6d revisions have incorrect parents in the revision index' %
                         len(checkobj.revs_with_bad_parents_in_index))

                dialogs._bzrInfoDialog("Repository check",checkreport,[200,300])
            except:
                dialogs._bzrErrorDialog("Bazaar error doing check.", sys.exc_value)
                    
         
    def on_bazaar_missing(self, menu):
        
        localpath = self._get_active_doc_dir()
        if localpath == None:
            dialogs._bzrErrorDialog("This action must be done from a saved document.")
        else:
            inputdlg = None
            try:
                br, relpath = branch.Branch.open_containing(localpath)
                   
                #rev_id = br.last_revision()
                #t = br.repository.revision_tree(rev_id)
                inputdlg = dialogs.BzrInputFileDialog("","Compare branchs","Select branch to compare with","Compare")
                ret, dest = inputdlg.run()
                inputdlg.disable_widgets()
                
                if ret == gtk.RESPONSE_OK and dest !='':
                    ######3
                    from bzrlib.missing import find_unmerged, iter_log_revisions
            
                    mine_only = None
                    theirs_only = None
                    restrict = 'all'
            
                    local_branch = br
                    other_branch = dest
                    inputdlg.set_status("Opening remote branch...")
                    if "ssh:" in bzrlib.bzrdir.get_transport(dest).base:
                        inputdlg.set_status("Opening SSH branch.\nYou may be prompted for your passphrase on the command line window")
                        time.sleep(0.05) # never could i make the dialog redraw correctly otherwise !
                        while gtk.events_pending(): gtk.main_iteration()

                    remote_branch = branch.Branch.open(other_branch)
                    if remote_branch.base == local_branch.base:
                        remote_branch = local_branch
                    local_branch.lock_read()
                    try:
                        remote_branch.lock_read()
                        try:
                            inputdlg.set_status("Comparing...")
                            local_extra, remote_extra = find_unmerged(
                                local_branch, remote_branch, restrict)
            
                            registry = log.log_formatter_registry
                            log_format = registry.get_default(local_branch)
                            
                            outstr = StringIO.StringIO()
                            lf = log_format(to_file=outstr,
                                            show_ids=None,
                                            show_timezone='original')
                            if local_extra is not None:
                                local_extra.reverse()
                            if remote_extra is not None:
                                remote_extra.reverse()
            
                            status_code = 0
                            if local_extra and not theirs_only:
                                outstr.write("You have %d extra revision(s):\n" %
                                                len(local_extra))
                                for revision in iter_log_revisions(local_extra,
                                                    local_branch.repository,
                                                    False):
                                    lf.log_revision(revision)
                                printed_local = True
                                status_code = 1
                            else:
                                printed_local = False
            
                            if remote_extra and not mine_only:
                                if printed_local is True:
                                    outstr.write("\n\n\n")
                                outstr.write("You are missing %d revision(s):\n" %
                                                len(remote_extra))
                                for revision in iter_log_revisions(remote_extra,
                                                    remote_branch.repository,
                                                    False):
                                    lf.log_revision(revision)
                                status_code = 1
            
                            if mine_only and not local_extra:
                                # We checked local, and found nothing extra
                                outstr.write('This branch is up to date.\n')
                            elif theirs_only and not remote_extra:
                                # We checked remote, and found nothing extra
                                outstr.write('Other branch is up to date.\n')
                            elif not (mine_only or theirs_only or local_extra or
                                      remote_extra):
                                # We checked both branches, and neither one had extra
                                # revisions
                                outstr.write("Branches are up to date.\n")
                        except:
                            dialogs._bzrErrorDialog("Bazaar error comparing.", sys.exc_value)
                        finally:
                            remote_branch.unlock()
                    except:
                        dialogs._bzrErrorDialog("Bazaar error comparing.", sys.exc_value)
                    finally:
                        local_branch.unlock()
                        dialogs._bzrInfoDialog("Compare branchs",outstr.getvalue(),[400,250])
                    if not status_code and parent is None and other_branch is not None:
                        local_branch.lock_write()
                        try:
                            # handle race conditions - a parent might be set while we run.
                            if local_branch.get_parent() is None:
                                local_branch.set_parent(remote_branch.base)
                        finally:
                            local_branch.unlock()
                    
                    ########
                elif ret == gtk.RESPONSE_OK and dest == '':
                    dialogs._bzrErrorDialog("Please enter another branch")
                
            except:
                dialogs._bzrErrorDialog("Bazaar error comparing.", sys.exc_value)
                
            if inputdlg != None:
                inputdlg.destroy()
                
    
    def on_bazaar_reconcile(self,menu):
        localpath = self._get_active_doc_dir()
        if localpath == None:
            dialogs._bzrErrorDialog("This action must be done from a saved document.")
        else:
            pid = subprocess.Popen(['%s/commands.py' % self._mypath, 'c="reconcile"', 'l="%s"' % localpath]).pid
    
    def on_bazaar_upgrade(self, menu):
        localpath = self._get_active_doc_dir()
        if localpath == None:
            dialogs._bzrErrorDialog("This action must be done from a saved document.")
        else:
            inputdlg = dialogs.BzrInputDialog("default","Upgrade branch format", "Enter branch format.","Branch format")
            retcode, format, options = inputdlg.run()
            if format == '':
                format = 'default'
            #doit = dialogs._bzrQuestionDialog("Upgrade?", "Are you sure you want to upgrade your branch storage format?")

            if retcode == gtk.RESPONSE_OK:
                uifactory.currentdialog = inputdlg
                

                try:
                    from bzrlib.upgrade import upgrade
                    format = bzrdir.format_registry.make_bzrdir(format)
                    upgrade(localpath, format)
                except:
                    msgerror = "Probably the format entered is not supported\n%s" % sys.exc_value
                    dialogs._bzrErrorDialog("Bazaar error upgrading. ", msgerror)
                
                inputdlg.destroy()
                uifactory.currentdialog = None
                
    def on_bazaar_pack(self, menu):
        localpath = self._get_active_doc_dir()
        if localpath == None:
            dialogs._bzrErrorDialog("This action must be done from a saved document.")
        else:
            doit = dialogs._bzrQuestionDialog("Pack?", "Are you sure you want to pack your branch?")

            if doit == gtk.RESPONSE_YES:
                inputdlg = dialogs.BzrProgressDialog("Pack branch")
                inputdlg.show_all()
                uifactory.currentdialog = inputdlg

                try:
                    dir = bzrdir.BzrDir.open_containing(localpath)[0]
                    try:
                        branch = dir.open_branch()
                        repository = branch.repository
                    except errors.NotBranchError:
                        repository = dir.open_repository()
                    repository.pack()
                except:
                    dialogs._bzrErrorDialog("Bazaar error packing.", sys.exc_value)
                
                inputdlg.destroy()
                uifactory.currentdialog = None
    
    def on_bazaar_breacklock(self, menu):
        localpath = self._get_active_doc_dir()
        if localpath == None:
            dialogs._bzrErrorDialog("This action must be done from a saved document.")
        else:
            try:
                control, relpath = bzrdir.BzrDir.open_containing(localpath)
            
                control.break_lock()
            except:
                dialogs._bzrErrorDialog("Bazaar error breacking log.", sys.exc_value)
                
    def on_bazaar_reconfigure(self, menu):
        localpath = self._get_active_doc_dir()
        if localpath == None:
            dialogs._bzrErrorDialog("This action must be done from a saved document.")
        else:
            values = {'branch':'Branch',
                      'checkout':'Checkout',
                      'lightweight-checkout':'Lightweight checkout',
                      'standalone':'Standalone',
                      'tree':'Tree',
                      'use-shared':'Use shared'
            }
            dlgoptions = {"--force":["Force even if local changes get lost",'B']}
            inputdlg = dialogs.BzrComboDialog(values, '',"Reconfigure branch", "Select reconfiguration type","Reconfigure branch",dlgoptions)
            ret, data, retoptions = inputdlg.run()
            force = dlgoptions['--force']
            uifactory.currentdialog = inputdlg
            if ret == gtk.RESPONSE_OK:
                try:
                    directory = bzrdir.BzrDir.open(localpath)
                    target_type = data
                    if target_type is None:
                        dialogs._bzrErrorDialog('No target configuration specified')
                    elif target_type == 'branch':
                        reconfiguration = reconfigure.Reconfigure.to_branch(directory)
                    elif target_type == 'tree':
                        reconfiguration = reconfigure.Reconfigure.to_tree(directory)
                    elif target_type == 'checkout':
                        bindtodialog = dialogs.BzrInputDialog("","Reconfigure branch to checkout", "Select bind location","Branch reconfiguration")
                        retcod, retdata = bindtodialog.run()
                        bindtodialog.destroy()
                        if retcod == gtk.RESPONSE_OK and retdata != '': 
                            reconfiguration = reconfigure.Reconfigure.to_lightweight_checkout(
                                                directory, retdata)
                        elif retcod == gtk.RESPONSE_OK and retdata == '':
                            dialogs._bzrErrorDialog("You need to enter a bind location")
                            
                        reconfiguration = reconfigure.Reconfigure.to_checkout(directory,
                                                                              retdata)
                    elif target_type == 'lightweight-checkout':
                        bindtodialog = dialogs.BzrInputDialog("","Reconfigure branch to checkout", "Select bind location","Branch reconfiguration")
                        retcod, retdata = bindtodialog.run()
                        bindtodialog.destroy()
                        if retcod == gtk.RESPONSE_OK and retdata != '': 
                            reconfiguration = reconfigure.Reconfigure.to_lightweight_checkout(
                                                directory, retdata)
                        elif retcod == gtk.RESPONSE_OK and retdata == '':
                            dialogs._bzrErrorDialog("You need to enter a bind location")
                    elif target_type == 'use-shared':
                        reconfiguration = reconfigure.Reconfigure.to_use_shared(directory)
                    elif target_type == 'standalone':
                        reconfiguration = reconfigure.Reconfigure.to_standalone(directory)
                    reconfiguration.apply(force)
                except:
                    dialogs._bzrErrorDialog("Bazaar error doing reconfiguration.", sys.exc_value)
                if inputdlg != None:
                    inputdlg.destroy()
                uifactory.currentdialog = None

              
    def on_bazaar_testament(self,menu):
        localpath = self._get_active_doc_dir()
        if localpath == None:
            dialogs._bzrErrorDialog("This action must be done from a saved document.")
        else:
            from bzrlib.testament import Testament

            testament_class = Testament
            b = branch.Branch.open_containing(localpath)[0]
            b.lock_read()
            try:
                rev_id = b.last_revision()
                t = testament_class.from_revision(b.repository, rev_id)
                data = t.as_short_text()
                dialogs._bzrInfoDialog("Branch testament",data,[400,250])
            except:
                dialogs._bzrErrorDialog("Bazaar error generating testament.", sys.exc_value)
            finally:
                b.unlock()
                   
    def on_bazaar_export(self,menu):  
        localpath = self._get_active_doc_dir()
        if localpath == None:
            dialogs._bzrErrorDialog("This action must be done from a saved document.")
        else:
            inputdlg = None
            try:
                br, relpath = branch.Branch.open_containing(localpath)
                   
                #rev_id = br.last_revision()
                #t = br.repository.revision_tree(rev_id)
                
                dlgoptions = {
                           "--revision":["Revision","R",br],
                           "--root":["Name of the root directory inside the exported file","T"]}
                
                inputdlg = dialogs.BzrInputFileDialog("","Export branch","Select export location\nSupported formats are tar, tgz, tbz2, zip and dir","Export",dlgoptions)
                ret, dest, ret_options = inputdlg.run()
                inputdlg.disable_widgets()
                #dest = "/home/javier/bzrt/export/xx.tgz"
                if ret == gtk.RESPONSE_OK and dest !='':
                    inputdlg.set_status("Exporting...")
                    from bzrlib.export import export
                    
                    opt_revision = ret_options['--revision']
                    opt_root = ret_options['--root']
                    
                    if opt_root == '':
                        opt_root = None

                    t = br.repository.revision_tree(opt_revision)
                    export(t, dest, None, opt_root)
                    
                elif ret == gtk.RESPONSE_OK and dest == '':
                    dialogs._bzrErrorDialog("Please enter a destination location")
                
            except bzrlib.errors.NoSuchExportFormat, e:
                dialogs._bzrErrorDialog('Unsupported export format', e.format)
            except:
                dialogs._bzrErrorDialog("Bazaar error doing export.", sys.exc_value)
                
            if inputdlg != None:
                inputdlg.destroy()
        
    def on_bazaar_conflicts(self,menu):
        if self._check_bzrgtk("conflicts"):
            localpath = self._get_active_doc_dir()
            
            if localpath == None:
                dialogs._bzrErrorDialog("This action must be done from a saved document.")
            else:
                try:
                    (wt, path) = workingtree.WorkingTree.open_containing(localpath)
                    from bzrlib.plugins.gtk.conflicts import ConflictsDialog
                    dialog = ConflictsDialog(wt)
                    dialog.run()    
                except:
                    dialogs._bzrErrorDialog("Bazaar error viewing conflicts.", sys.exc_value)

    def on_bazaar_patch(self,menu):
        if self._check_bzrgtk("patch"):
            localpath = self._get_active_doc_dir()
            if localpath == None:
                dialogs._bzrErrorDialog("This action must be done from a saved document.")
            else:
                try:
                    inputdlg = dialogs.BzrInputFileDialog("","Patch branch","Select patch file","Patch")
                    ret, patchfile = inputdlg.run()
                    if ret == gtk.RESPONSE_OK:
                        if patchfile == '':
                            dialogs._bzrErrorDialog("Bazaar error doing patch.", "You must select a file")
                        else:
                            from bzrlib.plugins.gtk.diff import (DiffController,
                                                                 MergeDirectiveController)
                            from bzrlib import merge_directive
                            lines = open(patchfile, 'rb').readlines()
                            lines = [l.replace('\r\n', '\n') for l in lines]
                            try:
                                directive = merge_directive.MergeDirective.from_lines(lines)
                            except bzrlib.errors.NotAMergeDirective:
                                controller = DiffController(path, lines)
                                window = controller.window
                            else:
                                mdirective = MergeDirectiveController(patchfile,directive)
                                window = mdirective.window
                            window.show()
                except:
                    dialogs._bzrErrorDialog("Bazaar error doing patch.", sys.exc_value)
                inputdlg.destroy()
                
    def on_bazaar_tags(self,menu):
        if self._check_bzrgtk("config"):
            localpath = self._get_active_doc_dir()
            if localpath == None:
                dialogs._bzrErrorDialog("This action must be done from a saved document.")
            else:
                try:
                    br, path = branch.Branch.open_containing(localpath)
                    from bzrlib.plugins.gtk.tags import TagsWindow
                    window = TagsWindow(br)
                    window.show()
                except:
                    dialogs._bzrErrorDialog("Bazaar error opening tags window.", sys.exc_value)                    
                    
                    
    def on_bazaar_config(self,menu):
        if self._check_bzrgtk("config"):
            localpath = self._get_active_doc_dir()
            if localpath == None:
                dialogs._bzrErrorDialog("This action must be done from a saved document.")
            else:
                try:
                    from bzrlib.plugins.gtk.preferences import PreferencesWindow
                    dialog = PreferencesWindow()
                    dialog.run()
                except:
                    dialogs._bzrErrorDialog("Bazaar error opening config.", sys.exc_value)                    
                    
    # Gedit callbacks
    def deactivate(self):
        self._remove_menu()
        self._remove_statusbar()

        self._window = None
        self._plugin = None
        self._action_group = None

    def update_ui(self):
        # Called whenever the window has been updated (active tab
        # changed, etc.)
        doc = self._window.get_active_document()

        if doc != None:
            docuri = doc.get_uri_for_display() # FIXME: is this the right one? or should i clean up get_uri?
            if docuri != None and os.path.exists(docuri):
                self._set_menu_sensitive(self._get_is_active_branch(docuri))
            else:
                self._set_menu_sensitive(False)
        else:
            self._set_menu_sensitive(False)

        self._statusbar_draw("")
                
    # Private ui functions
    

            
    def _insert_menu(self):
        # Get the GtkUIManager
        manager = self._window.get_ui_manager()

        # Create a new action group
        self._action_group = gtk.ActionGroup("BazaarActions")

        self._action_group.add_actions([("BazaarMenu", None, _("Bazaar"),
                                         None, _("Bazaar"),
                                         None)])

        #not bazaar dir
        self._action_group.add_actions([("BazaarInit", None, _("Create workspace"),
                                         None, _("Create workspace"),
                                         self.on_bazaar_init)])

        
        #bazaar dir        
        
        
        self._action_group.add_actions([("BazaarFile", None, _("Current _file"),
                                         None, _("Current file"),
                                         None)])
        
        self._action_group.add_actions([("BazaarAddAll", None, _("Add all files..."),
                                         None, _("Add all files..."),
                                         self.on_bazaar_add)])
                                         
        self._action_group.add_actions([("BazaarBranchInfo", None, _("_Branch info"),
                                         None, _("Branch info"),
                                         None)])
        self._action_group.add_actions([("BazaarBranchMaintenance", None, _("_Branch maintenance"),
                                         None, _("Branch maintenance"),
                                         None)])
        
        self._action_group.add_actions([("BazaarCommit", None, _("_Commit"),
                                         None, _("Commit"),
                                         self.on_bazaar_commit)])
        
        self._action_group.add_actions([("BazaarAdd", None, _("_Add files"),
                                         None, _("Add files"),
                                         None)])


        self._action_group.add_actions([("BazaarAddCurrent", None, _("Add _current file"),
                                         None, _("Add current file"),
                                         self.on_bazaar_add_current)])
        self._action_group.add_actions([("BazaarAddCurrentDir", None, _("Add files in current dir (non recursive)"),
                                         None, _("Add files in current dir (non recursive)"),
                                         self.on_bazaar_add_current_dir)])
                
        
        self._action_group.add_actions([("BazaarMVFile", None, _("_Move"),
                                         None, _("Move"),
                                         self.on_bazaar_move)])
                
        self._action_group.add_actions([("BazaarIgnoreCurrent", None, _("I_gnore"),
                                         None, _("Ignore"),
                                         self.on_bazaar_ignorecurrent)])
        self._action_group.add_actions([("BazaarRemoveCurrent", None, _("Remo_ve"),
                                         None, _("Remove"),
                                         self.on_bazaar_removecurrent)])
        
        self._action_group.add_actions([("BazaarPush", None, _("_Push to..."),
                                         None, _("Push to..."),
                                         self.on_bazaar_push)])
        
       

        #action = action.connect("activate",self.on_bazaar_log,'dir')
        
        
        self._action_group.add_actions([("BazaarPull", None, _("P_ull from..."),
                                         None, _("Pull from..."),
                                         self.on_bazaar_pull)])
        

        self._action_group.add_actions([("BazaarMerge", None, _("_Merge"),
                                         None, _("Merge"),
                                         self.on_bazaar_merge)])
        
        self._action_group.add_actions([("BazaarUpdate", None, _("Up_date"),
                                         None, _("Update"),
                                         self.on_bazaar_update)])
        self._action_group.add_actions([("BazaarStatus", None, _("_Status"),
                                         None, _("Status"),
                                         self.on_bazaar_status)])
        self._action_group.add_actions([("BazaarInfo", None, _("In_fo"),
                                         None, _("Info"),
                                         self.on_bazaar_info)])        
        
        self._action_group.add_actions([("BazaarAnnotate", None, _("Anno_tate"),
                                         None, _("Annotate"),
                                         self.on_bazaar_annotate)])
        
        
        self._action_group.add_actions([("BazaarSettings", None, _("Branch _settings"),
                                         None, _("Branch settings"),
                                         self.on_bazaar_branchsettings)])
       
        self._action_group.add_actions([("BazaarCreatePatch", None, _("_Patch"),
                                         None, _("Patch"),
                                         None)])
        self._action_group.add_actions([("BazaarSendPatch", None, _("Send..."),
                                         None, _("Send..."),
                                         None)])
        
        action = self._action_group.get_action("BazaarSendPatch")
        action = action.connect("activate",self.on_bazaar_send,'send')
        
        
        self._action_group.add_actions([("BazaarSavePatch", None, _("Save to file"),
                                         None, _("Save to file"),
                                         None)])        
        action = self._action_group.get_action("BazaarSavePatch")
        action = action.connect("activate",self.on_bazaar_send,'save')
                
        self._action_group.add_actions([("BazaarDiffCurrent", None, _("View _diff"),
                                         None, _("View _diff"),
                                         None)])
        action = self._action_group.get_action("BazaarDiffCurrent")
        action = action.connect("activate",self.on_bazaar_diff,'current')
        
        self._action_group.add_actions([("BazaarDiffBranch", None, _("View _diff"),
                                         None, _("View _diff"),
                                         None)])
        action = self._action_group.get_action("BazaarDiffBranch")
        action = action.connect("activate",self.on_bazaar_diff,'branch')
        
        
        self._action_group.add_actions([("BazaarLog", None, _("Show _log"),
                                         None, _("Show log"),
                                         None)])
        
        self._action_group.add_actions([("BazaarLogFile", None, _("View _log"),
                                         None, _("View log"),
                                         None)])
        action = self._action_group.get_action("BazaarLogFile")
        action = action.connect("activate",self.on_bazaar_log,'file')
        
        self._action_group.add_actions([("BazaarResolveFile", None, _("Mark _resolved"),
                                         None, _("Mark _resolved"),
                                         None)])        
        action = self._action_group.get_action("BazaarResolveFile")
        action = action.connect("activate",self.on_bazaar_resolve,'file')

        
        
        self._action_group.add_actions([("BazaarLogDir", None, _("_Directory log"),
                                         None, _("Directory log"),
                                         None)])
        action = self._action_group.get_action("BazaarLogDir")
        action = action.connect("activate",self.on_bazaar_log,'dir')
        
        self._action_group.add_actions([("BazaarLogBranch", None, _("_Branch log"),
                                         None, _("Branch log"),
                                         None)])
        

                                        
        action = self._action_group.get_action("BazaarLogBranch")
        action = action.connect("activate",self.on_bazaar_log,'branch')
        
        self._action_group.add_actions([("BazaarRevert", None, _("_Revert"),
                                         None, _("Revert"),
                                         self.on_bazaar_revert)])
        self._action_group.add_actions([("BazaarUncommit", None, _("_Uncommit"),
                                         None, _("Uncommit"),
                                         self.on_bazaar_uncommit)])

        
        self._action_group.add_actions([("BazaarViz", None, _("View _history"),
                                         None, _("View history"),
                                         self.on_bazaar_viz)])
        self._action_group.add_actions([("BazaarCheck", None, _("Chec_k"),
                                         None, _("Check"),
                                         self.on_bazaar_check)])
        self._action_group.add_actions([("BazaarMissing", None, _("Compare _with branch"),
                                         None, _("Compare _with branch"),
                                         self.on_bazaar_missing)])
        self._action_group.add_actions([("BazaarReconcile", None, _("_Reconcile"),
                                         None, _("Reconcile"),
                                         self.on_bazaar_reconcile)])
        
        self._action_group.add_actions([("BazaarResolve", None, _("_Resolve all"),
                                         None, _("_Resolve all"),
                                         None)])
        action = self._action_group.get_action("BazaarResolve")
        action = action.connect("activate",self.on_bazaar_resolve,'branch')
        
        self._action_group.add_actions([("BazaarUpgrade", None, _("_Upgrade"),
                                         None, _("_Upgrade"),
                                         self.on_bazaar_upgrade)])
        self._action_group.add_actions([("BazaarPack", None, _("_Pack"),
                                         None, _("Pack"),
                                         self.on_bazaar_pack)])
        
        self._action_group.add_actions([("BazaarBreackLock", None, _("_Break lock"),
                                         None, _("Break lock"),
                                         self.on_bazaar_breacklock)])
        
        self._action_group.add_actions([("BazaarReconfigure", None, _("_Reconfigure branch"),
                                         None, _("Reconfigure branch"),
                                         self.on_bazaar_reconfigure)])
        
        
        self._action_group.add_actions([("BazaarTestament", None, _("_View testament"),
                                         None, _("BazaarTestament"),
                                         self.on_bazaar_testament)])
        
        self._action_group.add_actions([("BazaarExport", None, _("E_xport"),
                                         None, _("Export"),
                                         self.on_bazaar_export)])
        
        

        self._action_group.add_actions([("BazaarConflicts", None, _("Vie_w conflicts"),
                                         None, _("View conflicts"),
                                         self.on_bazaar_conflicts)])
        self._action_group.add_actions([("BazaarPatch", None, _("Mana_ge patch"),
                                         None, _("Manage patch"),
                                         self.on_bazaar_patch)])
        self._action_group.add_actions([("BazaarTags", None, _("Manage _tags"),
                                         None, _("Manage tags"),
                                         self.on_bazaar_tags)])

        
        self._action_group.add_actions([("BazaarConfig", None, _("Bazaar Configuration"),
                                         None, _("Bazaar Configuration"),
                                         self.on_bazaar_config)])
        
        menutool = MenuToolAction("BazaarToolButton", "Bazaar",
                                          "Bazaar", None)
        self._action_group.add_action(menutool)


        # Insert the action group
        manager.insert_action_group(self._action_group, -1)

        # Merge the UI
        self._ui_id = manager.add_ui_from_string(ui_str)

        # create toolbuton with menu
        toolbuton = manager.get_widget("/ToolBar/BazaarToolButton")


        menu = manager.get_widget("/MenuBar/ToolsMenu/BazaarMenu/BazaarMenu")
        menuimg = gtk.Image()
        menuimg.set_from_file(self._mypath+"/img/bzr-icon-16.png")

        menu.set_image(menuimg)
        toolbuton.set_menu(menu.get_submenu())
        
        
        img = gtk.Image()
        img.set_from_file(self._mypath+"/img/icons/push16.png")
        manager.get_widget("/MenuBar/ToolsMenu/BazaarMenu/BazaarMenu/BazaarPush").set_image(img)

        img = gtk.Image()
        img.set_from_file(self._mypath+"/img/icons/commit16.png")
        manager.get_widget("/MenuBar/ToolsMenu/BazaarMenu/BazaarMenu/BazaarCommit").set_image(img)
                    
        img = gtk.Image()
        img.set_from_file(self._mypath+"/img/icons/diff16.png")
        manager.get_widget("/MenuBar/ToolsMenu/BazaarMenu/BazaarMenu/BazaarBranchInfo/BazaarDiffBranch").set_image(img)
        
        img = gtk.Image()
        img.set_from_file(self._mypath+"/img/icons/diff16.png")
        manager.get_widget("/MenuBar/ToolsMenu/BazaarMenu/BazaarMenu/BazaarFile/BazaarDiffCurrent").set_image(img)
                
        img = gtk.Image()
        img.set_from_file(self._mypath+"/img/icons/log16.png")
        manager.get_widget("/MenuBar/ToolsMenu/BazaarMenu/BazaarMenu/BazaarBranchInfo/BazaarLog").set_image(img)

        img = gtk.Image()
        img.set_from_file(self._mypath+"/img/icons/log16.png")
        manager.get_widget("/MenuBar/ToolsMenu/BazaarMenu/BazaarMenu/BazaarFile/BazaarLogFile").set_image(img)
        
        img = gtk.Image()
        img.set_from_file(self._mypath+"/img/icons/pull16.png")
        manager.get_widget("/MenuBar/ToolsMenu/BazaarMenu/BazaarMenu/BazaarPull").set_image(img)
        
        img = gtk.Image()
        img.set_from_file(self._mypath+"/img/icons/refresh16.png")
        manager.get_widget("/MenuBar/ToolsMenu/BazaarMenu/BazaarMenu/BazaarUpdate").set_image(img)
        
        
        #toolbuton.add_accelerator('activate',manager.get_accel_group(), ord('q'),"<Control>a",None)

        
    def _remove_menu(self):
        # Get the GtkUIManager
        manager = self._window.get_ui_manager()

        # Remove the ui
        manager.remove_ui(self._ui_id)

        # Remove the action group
        manager.remove_action_group(self._action_group)

        # Make sure the manager updates
        manager.ensure_update()

    def _remove_statusbar(self):
        gtk.Bin.remove(self._window.get_statusbar(),self._statusframe)
        # I spent 30min to get this f*****g thing done :)

    def _statusbar_draw(self,msg = ""):
        if self._statusframe == None:
            status = self._window.get_statusbar()
            self._statusframe = gtk.Frame()
            status.add(self._statusframe)
            hb = gtk.HBox()
            self.status_img = gtk.Image()

            self.status_img.set_from_file(self._mypath+"/img/bzr-icon-16_no.png")

            self._statuslbl = gtk.Label("")
            self._statuslbl.set_alignment(0,0)
            hb.pack_start(self.status_img)
            hb.pack_start(self._statuslbl)
            self._statusframe.add(hb)
            self._statusframe.show_all()

        if self._get_is_active_branch(self._get_active_doc_dir()) is not None:
            self.status_img.set_from_file(self._mypath+"/img/bzr-icon-16.png")
            if msg == "":
                msg = "This file is in an active bzr directory"
        else:
            if msg == "":
                msg = "This file isn't in an active bzr directory"
            self.status_img.set_from_file(self._mypath+"/img/bzr-icon-16_no.png")


        self._statuslbl.set_text(msg)

    def _set_menu_sensitive(self,isBranch):

        manager = self._window.get_ui_manager()
        toolbuton = manager.get_widget("/ToolBar/BazaarToolButton")
        img = gtk.Image()
        if isBranch:
            for menupath in self._menu_paths_bzr:
                manager.get_widget(menupath).set_sensitive(True)
            
            for menupath in self._menu_paths_nobzr:
                manager.get_widget(menupath).set_sensitive(False)            

            img.set_from_file(self._mypath+"/img/bzr-icon-24.png")

        else:
            for menupath in self._menu_paths_bzr:
                manager.get_widget(menupath).set_sensitive(False)
            
            for menupath in self._menu_paths_nobzr:
                manager.get_widget(menupath).set_sensitive(True)            

            img.set_from_file(self._mypath+"/img/bzr-icon-24_no.png")

        toolbuton.set_icon_widget(img)
        img.show()


    # Private data functions
               

            
    def _get_active_doc_dir(self):

        doc = self._window.get_active_document()
        if doc == None:
            return None
        else:
            docuri = doc.get_uri_for_display() # FIXME: is this the right one? or should i clean up get_uri?
            if os.path.exists(docuri):
                localpath = os.path.dirname(docuri)
                return localpath
            else:
                return None

    def _get_is_active_branch(self,dir):
        if not dir:
            return False
        try:
            br = branch.Branch.open_containing(dir)
        except:
            return False
        else:
            return True
        
    def _get_active_doc_uri_exists(self):
        doc = self._window.get_active_document()
        if doc != None:
            docuri = doc.get_uri_for_display()
            if os.path.exists(docuri):
                return docuri
            else:
                return None
        else:
            return None
        
    def _get_doc_from_uri(self, uri):
        app = gedit.app_get_default()
        docs = app.get_documents()
        for doc in docs:
            if doc.get_uri_for_display()  == uri:
                return doc
        return None
    
    def _check_bzrgtk(self,command):
        if not bzrgtk:
            dialogs._bzrErrorDialog("Can't run %s command. bzr-gtk plugin needs to be installed in " \
            "your system" % command,"Goto http://launchpad.net/bzr-gtk and install the plugin")
            return False
        else:
            return True

class BazaarPlugin(gedit.Plugin):
    def __init__(self):
        gedit.Plugin.__init__(self)
        self._instances = {}

    def activate(self, window):
        self._instances[window] = BazaarWindowHelper(self, window)

    def deactivate(self, window):
        self._instances[window].deactivate()
        del self._instances[window]

    def update_ui(self, window):
        self._instances[window].update_ui()
            
            


bzrlib.ui.ui_factory = uifactory.GtkUIFactory()

if __name__ == '__main__':
    print "cucu"