#! /usr/bin/env python
import sys
import dialogs

import bzrlib
from bzrlib import branch
from bzrlib import workingtree
from bzrlib import merge
from bzrlib import transport
from bzrlib import bzrdir
from bzrlib import revisionspec
from bzrlib.progress import DummyProgress
from bzrlib.plugin import load_plugins
load_plugins() #right now only used to make "branch lp:bla" work


import gtk
import uifactory

class BzrCommands(object):
    
    def __init__(self, command, args):
        for ar in args: 
            if ar[0] == 'l':
                local = ar[1]
                # we need to remove any " that is here
                if local[0] =='"':
                    local = local[1:]
                if local[-1] == '"':
                    local = local[:-1]
                
        if command == "push":            
            self.push(local)
        elif command == 'pull':
            self.pull(local)
        elif command == 'merge':
            self.merge(local)
        elif command == 'newworkspace':
            self.newworkspace()
        elif command == 'reconcile':
            self.reconcile(local)
  
    def newworkspace_cb(self, data, dialog):
        uifactory.currentdialog = dialog
        
        location,isrepo,createaction,branchname,parentlocation = data

        if isrepo:
            try:
                dialog.set_status("Creating shared repository...")
                format = bzrdir.format_registry.make_bzrdir('default')
                to_transport = transport.get_transport(location)
                to_transport.ensure_base()
        
                newdir = format.initialize_on_transport(to_transport)
                repo = newdir.create_repository(shared=True)
                repo.set_make_working_trees(True)
            except:
                dialogs._bzrErrorDialog("Bazaar creating repository.",sys.exc_value)
            else:
                if createaction != 'init':
                    location = os.path.join(location,branchname)


        if createaction == 'init' and not isrepo: #create empty branch
            try:
                dialog.set_status("Creating local branch")
                wt = bzrlib.bzrdir.BzrDir.create_standalone_workingtree(location)
            except:
                dialogs._bzrErrorDialog("Bazaar error executing Init",sys.exc_value)
        elif createaction == 'init-branch':
            try:
                dialog.set_status("Branching from remote location...")
                to_transport =  transport.get_transport(location)
                create_branch = bzrdir.BzrDir.create_branch_convenience
                create_branch(to_transport.base, format=None,
                                       possible_transports=[to_transport])
            except:
                dialogs._bzrErrorDialog("Bazaar error creating branch in shared repository",sys.exc_value)

        elif createaction == 'branch':
            try:
                dialog.set_status("Opening remote branch...")
                onlinebranch = branch.Branch.open(parentlocation)
                dialog.set_status("Creating local branch...")
                localbranch = onlinebranch.bzrdir.sprout(location)
            except:
                dialogs._bzrErrorDialog("Bazaar error branching",sys.exc_value)

        elif createaction == 'checkout':
            try:
                dialog.set_status("Opening remote location...")
                acel_trans, onlinebranch = bzrdir.BzrDir.open_tree_or_branch(parentlocation)
                dialog.set_status("Creating local checkout...")
                onlinebranch.create_checkout(location,None,False,acel_trans,False)
            except :
                dialogs._bzrErrorDialog("Bazaar error doing checkout",sys.exc_value)
                
        uifactory.currentdialog = None

                
    def merge(self, localpath):
        try:
            wt,relpath = workingtree.WorkingTree.open_containing(localpath)
        except:
            dialogs._bzrErrorDialog("Bazaar error opening %s" % localpath ,sys.exc_value)
        else:
            mergeloc = wt.branch.get_parent()
            if mergeloc == None:
                mergeloc = ''
                
            dlgoptions = {"--pull":["Pull",'B'],
                       "--remember": ["Remember location","B"],
                       "--force":["Force even with uncommited changes",'B'],
                       "--reprocess":["Reprocess to reduce spurious conflicts.",'B'],
                       "--uncommitted":["Apply uncommitted changes.",'B'],
                       "--change":["Select changes revision","T"],
                       "--revision":["Revision","R",wt.branch]}
            
            inputdlg = dialogs.BzrInputDialog(mergeloc,"Merge from", "Enter the location from where you want to merge code","Merge",dlgoptions)
            uifactory.currentdialog = inputdlg
            response, mergelocation, retoptions = inputdlg.run()
            inputdlg.disable_widgets()
            
            opt_remember = retoptions['--remember']
            opt_change = retoptions['--change']
            opt_revision = retoptions['--revision']
            opt_reprocess = retoptions['--reprocess']
            opt_uncommitted = retoptions['--uncommitted']
            opt_pull = retoptions['--pull']
            opt_force = retoptions['--force']
            
                

            # FIXME: this dialog yould allow to force "update" and ?to save new location?
            # and select revision, allow merge bundles...
            if response == gtk.RESPONSE_OK and mergelocation != '':
                try:
                    inputdlg.set_status("Opening remote branch...")
                    
                    branch_remote = branch.Branch.open(mergelocation)
                    if opt_change != "":
                        opt_change_id = revisionspec.RevisionSpec.from_string(opt_change).as_revision_id(branch_remote)
                    else:
                        opt_change_id = branch_remote.last_revision()
                except:
                    dialogs._bzrErrorDialog("Bazaar error opening %s" % mergelocation ,sys.exc_value)
                else:
                    try:
                        inputdlg.set_status("Merging...")
                        
                        if opt_uncommitted and opt_revision != '' and opt_revision != wt.branch.last_revision():
                            dialogs._bzrErrorDialog("Can't do --uncommited with --revision")
                        else:
                            if opt_remember:
                                wt.branch.set_submit_branch(branch_remote.base)
                                
                            if opt_revision != '':
                                opt_revision = wt.branch.last_revision()
                            
                            if opt_uncommitted:
                                remotewt, remoterel = workingtree.WorkingTree.open_containing(mergelocation)
                                merger = merge.Merger.from_uncommitted(wt, remotewt,
                                                                       DummyProgress())
                            else:
                                merger = merge.Merger.from_revision_ids(DummyProgress(),wt,opt_change_id,opt_revision)
                            
                            merger.check_basis(not opt_force)
                            
                            if opt_pull:
                                if merger.interesting_files is not None:
                                    dialogs._bzrErrorDialog("Cannot pull individual files")
                                    return False
    
                                if (merger.base_rev_id == wt.last_revision()):
                                    result = wt.pull(merger.other_branch, False,
                                                       merger.other_rev_id)
                            else:
                                merger.merge_type = merge.Merge3Merger
                                merger.reprocess = opt_reprocess
                                if merger.other_rev_id == merger.base_rev_id:
                                    dialogs._bzrErrorDialog("Can't merge. Nothing to do")
                                else:
                                    ret = merger.do_merge()
                                    if ret != 0:
                                        dialogs._bzrErrorDialog("This merge caused conflicts")
                                
                    except:
                        dialogs._bzrErrorDialog("Bazaar error merging from %s" % mergelocation ,sys.exc_value)

                        # FIXME: should force update?
            elif response == gtk.RESPONSE_OK and mergelocation == '':
                dialogs._bzrErrorDialog("Error. You have to enter a merge location.")
            inputdlg.destroy()
            uifactory.currentdialog = None
            
    def pull(self,localpath):
        try:
            br,relpath = branch.Branch.open_containing(localpath)
        except:
            dialogs._bzrErrorDialog("Bazaar error opening %s" % localpath ,sys.exc_value)
        else:
            pullloc = br.get_parent()
            if pullloc == None:
                pullloc = ''
                
            dlgoptions = {"--remember":["Remember location",'B'],
                       "--overwrite": ["Overwrite remote branch","B"],
                       "--revision":["Revision","T"]}
            
            inputdlg = dialogs.BzrInputDialog(pullloc,"Pull from parent", "Enter the location from where you want to pull code","Pull", dlgoptions)
            #currentdlg = inputdlg
            uifactory.currentdialog = inputdlg
            response, pulllocation, retoptions = inputdlg.run()
            
            remember = retoptions['--remember']
            overwrite = retoptions['--overwrite']
            revision = retoptions['--revision']
            
            
            inputdlg.disable_widgets()
            # FIXME: this dialog yould allow to force "update" and ?to save new location?
            if response == gtk.RESPONSE_OK and pulllocation != '':
                try:
                    inputdlg.set_status("Opening remote branch...")
                    branch_remote = branch.Branch.open(pulllocation)
                except:
                    dialogs._bzrErrorDialog("Bazaar error opening %s" % pulllocation ,sys.exc_value)
                else:
                    try:
                        if revision != "":
                            revision_id = revisionspec.RevisionSpec.from_string(revision).as_revision_id(branch_remote)
                        else:
                            revision_id = None

                        inputdlg.set_status("Pulling...")
                        try: # this try is needed because somehow branch.pull doesn't retrieve files anymore :(
                            tree_to = workingtree.WorkingTree.open_containing(localpath)[0]
                            branch_to = tree_to.branch
                        except bzrlib.errors.NoWorkingTree:
                            tree_to = None
                            branch_to = branch.Branch.open_containing(localpath)[0]
                        if tree_to != None:
                            res = tree_to.pull(branch_remote,overwrite,
                                               stop_revision=revision_id)
                        else:
                            res = branch_to.pull(branch_remote,overwrite,
                                               stop_revision=revision_id)
                            
                        if remember:
                            if br.is_locked():
                                br.unlock()
                            br.set_pull_location(pulllocation)
                        branch_dest.bzrdir.open_workingtree().update()
                    except:
                        dialogs._bzrErrorDialog("Bazaar error pulling from %s" % pulllocation ,sys.exc_value)
                    else:
                        dialogs._bzrErrorDialog("OK", "OK")
            elif response == gtk.RESPONSE_OK and pulllocation == '':
                dialogs._bzrErrorDialog("Error. You have to enter a pull location.")
            inputdlg.destroy()
            uifactory.currentdialog = None
                
    def push(self,localpath):     
        try:
            br,relpath = branch.Branch.open_containing(localpath)
        except:
            dialogs._bzrErrorDialog("Bazaar error pushing %s" % localpath,sys.exc_value)
        else:
            pushloc = br.get_push_location()
            if pushloc == None:
                pushloc = ''
            dlgoptions = {"--remember":["Remember location",'B'],
                       "--overwrite": ["Overwrite remote branch","B"],
                       "--revision":["Push selected revision","R",br]}
            inputdlg = dialogs.BzrInputDialog(pushloc,"Push your changes", "Enter the location where you want to push your branch","Push",dlgoptions)
            uifactory.currentdialog = inputdlg
            response, pushlocation,retoptions = inputdlg.run()
            
            remember = retoptions['--remember']
            overwrite = retoptions['--overwrite']
            revision = retoptions['--revision']

            inputdlg.disable_widgets()
            # FIXME: previous dialog yould also allow to save new location...
            if response == gtk.RESPONSE_OK and pushlocation != '':
                
                # Fix to bug #213515
                # first, get a transport
                to_transport = transport.get_transport(pushlocation)
                # then, test if there's a dir in remote location
                
                inputdlg.set_status("Opening remote location")
                remote_branch_ok = False
                try:
                    branch_dest = branch.Branch.open(pushlocation)
                except bzrlib.errors.NotBranchError:
                    inputdlg.set_status("Creating dir in remote location")
                    # it doesn't exist. Create.
                    try:
                        br.bzrdir.clone_on_transport(to_transport)
                        # FIXME: should add revision_id 
                    except:
                        dialogs._bzrErrorDialog("Bazaar error creating remote branch(%s)" % pushlocation,sys.exc_value)
                    else:
                        inputdlg.set_status("Opening remote location")
                        try:
                            branch_dest = branch.Branch.open(pushlocation)
                        except:
                            dialogs._bzrErrorDialog("Bazaar error opening destination branch(%s)" % pushlocation,sys.exc_value)                            
                        else:
                            remote_branch_ok = True
                except:
                    dialogs._bzrErrorDialog("Bazaar error opening destination branch(%s)" % pushlocation,sys.exc_value)
                else:
                    remote_branch_ok = True
                
               
                if remote_branch_ok:
                    
                    inputdlg.set_status("Pushing...")
                    res = br.push(branch_dest,overwrite,
                                               stop_revision=revision)
                    # FIXME: error if diverged, should show dialog
                    if remember:
                        if br.is_locked():
                            br.unlock()
                        br.set_push_location(pushlocation)
                    branch_dest.bzrdir.open_workingtree().update()
                        
                    #except:
                    #    dialogs._bzrErrorDialog("Bazaar error pushing" ,sys.exc_value)
                    #else:
                    #    self._statusbar_draw("Branch pushed to %s" % pushlocation)
                        
            elif pushlocation == '' and response == gtk.RESPONSE_OK:
                dialogs._bzrErrorDialog("Error. You have to enter a push location.")
            inputdlg.destroy()
            uifactory.currentdialog = None
            
    def newworkspace(self):
        dlgnwp = dialogs.BzrNewWorkspace(self.newworkspace_cb)
        dlgnwp.show()
        gtk.main()
    
    def reconcile(self,path):
        
        inputdlg = dialogs.BzrProgressDialog("Reconcile branch")
        inputdlg.show_all()
        uifactory.currentdialog = inputdlg
        from bzrlib.reconcile import reconcile
        
        dir = bzrdir.BzrDir.open(path)

        reconcile(dir)
        
        inputdlg.destroy()
        uifactory.currentdialog = None
            
if __name__ == '__main__':
    
    
    
    bzrlib.ui.ui_factory = uifactory.GtkUIFactory()
    cmds = []
    for c in sys.argv:
        cmd = c.split("=")[0]
        opt = c[len(cmd)+1:]
        if cmd == 'c':
            command = opt[1:-1]
        elif opt != '':
            cmds.append((cmd,opt)) 

    command = "merge"
    cmds = []
    cmds.append(('l','/home/javier/bzrt/xxx'))
    x = BzrCommands(command, cmds)
