from gtk import TreeStore, GenericTreeModel
from bzrlib.diff import show_diff_trees
from cStringIO import StringIO

FILE_STATUS_UNKNOWN = 0
FILE_STATUS_ADDED = 1
FILE_STATUS_MODIFIED = 2
FILE_STATUS_UNCHANGED = 3

TREE_MODIFIED = 1

class BzrEntryRowProxy(list):
    __row_mapping__ = {"icon_name":0, "display_path":1, "uri":2, "status_icon":3, "visible":4, "selected":5, "status":6, "filename":7}

    def __init__(self, row):
        list.__init__(self, row)
        self._row = row

    def __getattr__(self, name):
        index = self.__row_mapping__.get(name, -1)
        if index == -1:
            raise AttributeError("no attribute named '%s'"%(name))
        return self._row[index]

    def __setattr__(self, name, value):
        index = self.__row_mapping__.get(name, -1)
        if index == -1:
            list.__setattr__(self, name, value)
            return
        self._row[index] = value

class BzrEntriesTreeStore(TreeStore):
    def __init__(self, wt):
        TreeStore.__init__(self, str, str, str, str, bool, bool, int, str, str)
        # add the root
        self._root_iter = self.append(None)
        self._filter = ""
        self._modelfilter = None
        self.tree_state = 0
        self.wt = wt

    def append_entry(self, parent, *args):
        return self.append(parent, args)

    def get_root_iter(self):
        return self._root_iter

    def get_filter_text(self):
        return self._filter

    def set_filter_text(self, filter):
        self._filter = filter

    def get_diff_text(self):
        s = StringIO()
        show_diff_trees(self.wt.basis_tree(), self.wt, s, specific_files=None,
                        old_label='', new_label='',)
        return s.getvalue()

    def filter_new(self):
        modelfilter = TreeStore.filter_new(self, root="0")
        modelfilter.set_visible_column(4)
        return modelfilter

    def __getitem__(self, key):
        row = TreeStore.__getitem__(self, key)
        return BzrEntryRowProxy(row)

