import os, re
import os.path

import gobject
import gtk
import pango
import time

from bzrlib.plugins.gtk import gettext
_i18n = gettext.gettext

from bzrlib import workingtree, inventory, errors, branch, osutils
from bzrlib.tsort import merge_sort
from bzrlib.revision import NULL_REVISION

from bzrlib.errors import NotBranchError

#import gedit, gtksourceview2

class FileListTreeView(gtk.TreeView):
    
    def __init__(self, path = None, fileclickcb = None):
        gtk.TreeView.__init__(self)
        
        self.fileclickcb = fileclickcb
        
        self.set_property("headers-visible", True)
        self.set_property("rules-hint", True)
        self.get_selection().set_mode(gtk.SELECTION_SINGLE)
        self.connect("row-activated", self._row_activated)
        
        self._tvcolumn_filename = gtk.TreeViewColumn(_i18n('Filename'))
        self._tvcolumn_status = gtk.TreeViewColumn(_i18n('Status'))
        self._tvcolumn_size = gtk.TreeViewColumn(_i18n('Size'))
        self._tvcolumn_mtime = gtk.TreeViewColumn(_i18n('Last modified'))
        self.append_column(self._tvcolumn_filename)
        self.append_column(self._tvcolumn_status)
        self.append_column(self._tvcolumn_size)
        self.append_column(self._tvcolumn_mtime)
        
        
        # Set up the cells
        cellpb = gtk.CellRendererPixbuf()
        cell = gtk.CellRendererText()
        self._tvcolumn_filename.pack_start(cellpb, False)
        self._tvcolumn_filename.pack_start(cell, True)
        self._tvcolumn_filename.set_attributes(cellpb, stock_id=0)
        self._tvcolumn_filename.add_attribute(cell, 'text', 2)
        self._tvcolumn_status.pack_start(cell, True)
        self._tvcolumn_status.add_attribute(cell, 'text', 3)
        self._tvcolumn_size.pack_start(cell, True)
        self._tvcolumn_size.add_attribute(cell, 'text', 6)
        self._tvcolumn_mtime.pack_start(cell, True)
        self._tvcolumn_mtime.add_attribute(cell, 'text', 8)
        
        # Set up the properties of the TreeView
        self.set_headers_visible(True)
        self.set_headers_clickable(True)
        self.set_search_column(1)
        self._tvcolumn_filename.set_resizable(True)
        self._tvcolumn_status.set_resizable(True)
        self._tvcolumn_size.set_resizable(True)
        self._tvcolumn_mtime.set_resizable(True)
        self._tvcolumn_filename.set_sort_column_id(13)
        self._tvcolumn_status.set_sort_column_id(3)
        self._tvcolumn_size.set_sort_column_id(5)
        self._tvcolumn_mtime.set_sort_column_id(7)
        
        if path != None:
            self.set_path(path, True)
        
    def _row_activated(self, path, view_column, param):
        selpath = os.path.abspath(self.path + self.get_selected())
        
        if os.path.isdir(selpath):
            self.set_path(selpath,True)
        else:
            self.fileclickcb(selpath)
            #open file
        
    def get_selected(self):
        """ Get the selected filename. """
        treeselection = self.get_selection()
        (model, iter) = treeselection.get_selected()
        
        if iter is None:
            return None
        else:
            return model.get_value(iter, 2)
        
    def set_path(self, path, update = True):
        if path[-1] != '/':
            path = path + "/"
        try:
            self.wt, relpath = workingtree.WorkingTree.open_containing(path)
        except:
            self.clear_tree()
        else:
            self.path = path
            if update:
                self.draw_tree()
    
    def clear_tree(self):
        liststore = gtk.ListStore(gobject.TYPE_STRING)
        self.set_model(liststore)
    
    def update_file(self, file):
        """model = self.get_model()
        for row in model:
            if self.path + model.get_value(row.iter, 2) == file:"""
        self.draw_tree()
                
    def draw_tree(self):
        # Create ListStore
        # Model: [ icon, dir, name, status text, status, size (int), size (human), mtime (int), mtime (local), fileid ]
        wt = self.wt
        
        liststore = gtk.ListStore(gobject.TYPE_STRING,
                                  gobject.TYPE_BOOLEAN,
                                  gobject.TYPE_STRING,
                                  gobject.TYPE_STRING,
                                  gobject.TYPE_STRING,
                                  gobject.TYPE_STRING,
                                  gobject.TYPE_STRING,
                                  gobject.TYPE_INT,
                                  gobject.TYPE_STRING,
                                  gobject.TYPE_STRING)
        
        dirs = []
        files = []
        if wt.id2abspath(wt.get_root_id()) != self.path: #if we are in root, we can't go lower
            dirs.append("..")
        
        # Fill the appropriate lists
        dotted_files = True
        for item in os.listdir(self.path):
            if not dotted_files and item[0] == '.':
                continue
            if os.path.isdir(self.path + os.sep + item):
                dirs.append(item)
            else:
                files.append(item)
        
        branch = wt.branch
        tree2 = wt.branch.repository.revision_tree(branch.last_revision())
    
        delta = wt.changes_from(tree2, want_unchanged=True)
        
        # Add'em to the ListStore
        for item in dirs:
            try:
                statinfo = os.stat(self.path + os.sep + item)
            except OSError, e:
                if e.errno == 40:
                    continue
                else:
                    raise
            liststore.append([ gtk.STOCK_DIRECTORY,
                               True,
                               item,
                               '',
                               '',
                               "<DIR>",
                               "<DIR>",
                               statinfo.st_mtime,
                               self._format_date(statinfo.st_mtime),
                               ''])
            
        deltafiles = {}
        currpath = self.path.replace(wt.id2abspath(wt.get_root_id()),"")
        try:
            wt.lock_read()
            for rpath, id, kind in delta.unchanged:
                deltafiles.update({rpath : [id,'unchanged']})
            for rpath, id, kind, text_modified, meta_modified in delta.modified:
                deltafiles.update({rpath : [id,'modified']})
            for rpath, rpathnew, id, kind, text_modified, meta_modified in delta.renamed:
                deltafiles.update({rpath : [id,'renamed']})
            for rpath, id, kind in delta.added:
                deltafiles.update({rpath : [id,'added']})
            for rpath, id, kind in delta.removed:
                deltafiles.update({rpath : [id,'removed']})
            for rpath, file_class, kind, id, entry in wt.list_files():
                if not deltafiles.has_key(rpath) and file_class == 'I':
                    deltafiles.update({rpath : [id,'ignored']})
                elif not deltafiles.has_key(rpath):
                    deltafiles.update({rpath : [id,'unknow']})
        finally:
            wt.unlock()
        
        pathfiles = {}
        for item in files:
            filename = wt.relpath(self.path + os.sep + item)
            if deltafiles.has_key(filename):
                fileid = deltafiles[filename][0]
                st = deltafiles[filename][1]
            else:
                st = 'unknow'
                fileid = None
            status = st
            
        
            try:
                statinfo = os.stat(self.path + os.sep + item)
            except OSError, e:
                if e.errno == 40:
                    continue
                else:
                    raise
            liststore.append([gtk.STOCK_FILE,
                              False,
                              item,
                              st,
                              status,
                              str(statinfo.st_size), # NOTE: if int used there it will fail for large files (size expressed as long int)
                              self._format_size(statinfo.st_size),
                              statinfo.st_mtime,
                              self._format_date(statinfo.st_mtime),
                              fileid])

        # Create the columns and add them to the TreeView
        self.set_model(liststore)

        # Set up sorting
        liststore.set_sort_func(13, self._sort_filelist_callback, None)
        liststore.set_sort_column_id(13, gtk.SORT_ASCENDING)


        #self.show_all()
    def _sort_filelist_callback(self, model, iter1, iter2, data):
        """ The sort callback for the file list, return values:
        -1: iter1 < iter2
        0: iter1 = iter2
        1: iter1 > iter2
        """
        name1 = model.get_value(iter1, 2)
        name2 = model.get_value(iter2, 2)
        
        if model.get_value(iter1, 1):
            # item1 is a directory
            if not model.get_value(iter2, 1):
                # item2 isn't
                return -1
            else:
                # both of them are directories, we compare their names
                if name1 < name2:
                    return -1
                elif name1 == name2:
                    return 0
                else:
                    return 1
        else:
            # item1 is not a directory
            if model.get_value(iter2, 1):
                # item2 is
                return 1
            else:
                # both of them are files, compare them
                if name1 < name2:
                    return -1
                elif name1 == name2:
                    return 0
                else:
                    return 1
                
    def _format_date(self, timestamp):
        """ Format the time (given in secs) to a human readable format. """
        return time.ctime(timestamp)
    def _format_size(self, size):
        """ Format size to a human readable format. """
        if size < 1000:
            return "%d[B]" % (size,)
        size = size / 1000.0
        
        for metric in ["kB","MB","GB","TB"]:
            if size < 1000:
                break
            size = size / 1000.0
        return "%.1f[%s]" % (size,metric) 
    

class RevisionCombo(gtk.ComboBox):
    def __init__(self, branch, selected=None):
        gtk.ComboBox.__init__(self)
        model = gtk.ListStore(str,str)
        self.set_model(model)
        
        self.branch = branch
        branch.lock_read()
        graph = self.branch.repository.get_graph()
        graph_parents = {}
        for i, (revid, parent_revids) in enumerate(graph.iter_ancestry([branch.last_revision()])):
            if parent_revids == (NULL_REVISION,):
                graph_parents[revid] = ()
            else:
                graph_parents[revid] = parent_revids
        
        graph_parents["top:"] = [branch.last_revision()]

        merge_sorted_revisions = merge_sort(
            graph_parents,
            "top:",
            generate_revno=True)

        model.append((":last","Last revision"))
        for item in merge_sorted_revisions:
            numver = ""
            for ver in item[3]:
                numver = numver + str(ver) + "."
            numver = numver[0:-1]
            model.append((item[1],numver))
            
        cell = gtk.CellRendererText()
        self.pack_start(cell, True)
        self.add_attribute(cell, 'text', 1)
        if selected != None:
            self.set_active(selected)  
        else:
            self.set_active(0)
            
    def get_value(self):
        act = self.get_model()[self.get_active()]
        if act != None:
            if act[0] == ':last':
                return self.branch.last_revision()
            else:
                return act[0]
        else:
            return None


class OptionsBox(gtk.Expander):
    opt_widgets = {}
    
    def __init__(self, options):
        """
        Options is a dictionary containing the anything as a key
        and a list as it's data.
        The list contains the option name as first item and
        option type as seccond item.
        Option types are:
            'B': for boolean
            'T': for single line text
            'R': for revision
                This type of option needs a branch as a third item 
                
        """
        gtk.Expander.__init__(self, "Options")
        self.options = options
        self.vbox = gtk.VBox()
        self.add(self.vbox)
        self._draw_options()
        
    def _draw_options(self):
        options = self.options
        for op in options:
            if options[op][1] == 'B':
                check = gtk.CheckButton(options[op][0])
                wdgt = check
                wdgt.datacontrol = wdgt
                self.opt_widgets[op] = wdgt
                self.vbox.pack_start(wdgt,False,False)
            elif options[op][1] == 'T':
                entry = gtk.Entry()
                wdgt = gtk.Expander(options[op][0])
                wdgt.datacontrol = entry
                wdgt.add(entry)
                self.opt_widgets[op] = wdgt
                self.vbox.pack_start(wdgt,False,False)
            elif options[op][1] == 'R':
                cmb = RevisionCombo(options[op][2])
                wdgt = gtk.Expander(options[op][0])
                wdgt.datacontrol = cmb
                wdgt.add(cmb)
                self.opt_widgets[op] = wdgt
                self.vbox.pack_start(wdgt,False,False)
                
    def get_options(self):
        options = self.options
        data = {}
        for op in options:
            obj = self.opt_widgets[op].datacontrol
            if options[op][1] == 'B':
                data[op] = obj.get_active()
            elif options[op][1] == 'T':
                data[op] = obj.get_text()
            elif options[op][1] == 'R':
                data[op] = obj.get_value()
        
        return data
                


class TreeChanges(gtk.TreeView):
    def __init__(self,wt):
        gtk.TreeView.__init__(self)
        self._wt = wt
        
        crt = gtk.CellRendererToggle()
        commit_col = gtk.TreeViewColumn("Commit", crt, active=2)
        
        self.append_column(commit_col)
        self.append_column(gtk.TreeViewColumn(('Path'),
                                           gtk.CellRendererText(), text=3))
        self.append_column(gtk.TreeViewColumn(('Type'),
                                           gtk.CellRendererText(), text=4))
        crt.connect("toggled", self._toggle_commit)
        
        self.reload()
    
    def _toggle_commit(self, cell, path):
        model = self.get_model()
        
        if model[path][0] is None: # No file_id means 'All Files'
            new_val = not model[path][2]
            for node in model:
                node[2] = new_val
        else:
            model[path][2] = not model[path][2]
            if not model[path][2]: # if a file is deselected, deselect "all files"
                model[self.initial_cursor][2] = False
            

    def reload(self):
        liststore = gtk.ListStore(
            gobject.TYPE_STRING,  # [0] file_id
            gobject.TYPE_STRING,  # [1] real path
            gobject.TYPE_BOOLEAN, # [2] checkbox
            gobject.TYPE_STRING,  # [3] display path
            gobject.TYPE_STRING,  # [4] changes type
            gobject.TYPE_STRING,  # [5] commit message
            )
        
        self._selected = None
        store = liststore
        self.set_model(None)

        added = ('added')
        removed = ('removed')
        renamed = ('renamed')
        renamed_and_modified = ('renamed and modified')
        modified = ('modified')
        kind_changed = ('kind changed')

        # The store holds:
        # [file_id, real path, checkbox, display path, changes type, message]
        # iter_changes returns:
        # (file_id, (path_in_source, path_in_target),
        #  changed_content, versioned, parent, name, kind,
        #  executable)

        all_enabled = True
        # The first entry is always the 'whole tree'
        all_iter = store.append([None, None, all_enabled, 'All Files', '', ''])
        initial_cursor = store.get_path(all_iter)
        self.initial_cursor = initial_cursor # save all files path 
        # should we pass specific_files?
        self._wt.lock_read()
        self._wt.basis_tree().lock_read()
        try:
            
            for (file_id, real_path, change_type, display_path
                ) in self.iter_changes_to_status(self._wt.basis_tree(), self._wt):
                if self._selected and real_path != self._selected:
                    enabled = False
                else:
                    enabled = True
                item_iter = store.append([
                    file_id,
                    real_path.encode('UTF-8'),
                    enabled,
                    display_path.encode('UTF-8'),
                    change_type,
                    '', # Initial comment
                    ])
                if self._selected and enabled:
                    initial_cursor = store.get_path(item_iter)
        finally:
            self._wt.basis_tree().unlock()
            self._wt.unlock()

        self.set_model(store)
        
    def get_specific_files(self):
        """Return the list of selected paths, and file info.

        :return: ([unicode paths], [{utf-8 file info}]
        """
        
        files = []
        records = iter(self.get_model())
        rec = records.next() # Skip the All Files record
        assert rec[0] is None, "Are we skipping the wrong record?"

        file_info = []
        for record in records:
            if record[2]:# [2] checkbox
                file_id = record[0] # [0] file_id
                path = record[1]    # [1] real path
                file_message = record[5] # [5] commit message

                files.append(path.decode('UTF-8'))
                if file_message:
                    # All of this needs to be utf-8 information
                    file_info.append({'path':path, 'file_id':file_id,
                                     'message':file_message})
        file_info.sort(key=lambda x:(x['path'], x['file_id']))
        return files, file_info
        
        
    def iter_changes_to_status(self, source, target):
        """Determine the differences between trees.
    
        This is a wrapper around iter_changes which just yields more
        understandable results.
    
        :param source: The source tree (basis tree)
        :param target: The target tree
        :return: A list of (file_id, real_path, change_type, display_path)
        """
        added = 'added'
        removed = 'removed'
        renamed = 'renamed'
        renamed_and_modified = 'renamed and modified'
        modified = 'modified'
        kind_changed = 'kind changed'
    
        # TODO: Handle metadata changes
    
        status = []
        target.lock_read()
        try:
            source.lock_read()
            try:
                for (file_id, paths, changed_content, versioned, parent_ids, names,
                     kinds, executables) in target.iter_changes(source):
    
                    # Skip the root entry if it isn't very interesting
                    if parent_ids == (None, None):
                        continue
    
                    change_type = None
                    if kinds[0] is None:
                        source_marker = ''
                    else:
                        source_marker = osutils.kind_marker(kinds[0])
                    if kinds[1] is None:
                        assert kinds[0] is not None
                        marker = osutils.kind_marker(kinds[0])
                    else:
                        marker = osutils.kind_marker(kinds[1])
    
                    real_path = paths[1]
                    if real_path is None:
                        real_path = paths[0]
                    assert real_path is not None
                    display_path = real_path + marker
    
                    present_source = versioned[0] and kinds[0] is not None
                    present_target = versioned[1] and kinds[1] is not None
    
                    if present_source != present_target:
                        if present_target:
                            change_type = added
                        else:
                            assert present_source
                            change_type = removed
                    elif names[0] != names[1] or parent_ids[0] != parent_ids[1]:
                        # Renamed
                        if changed_content or executables[0] != executables[1]:
                            # and modified
                            change_type = renamed_and_modified
                        else:
                            change_type = renamed
                        display_path = (paths[0] + source_marker
                                        + ' => ' + paths[1] + marker)
                    elif kinds[0] != kinds[1]:
                        change_type = kind_changed
                        display_path = (paths[0] + source_marker
                                        + ' => ' + paths[1] + marker)
                    elif changed_content is True or executables[0] != executables[1]:
                        change_type = modified
                    else:
                        assert False, "How did we get here?"
    
                    status.append((file_id, real_path, change_type, display_path))
            finally:
                source.unlock()
        finally:
            target.unlock()
    
        return status


if __name__ == '__main__':
    def clik(but):
        print ob.get_options()
    wnd = gtk.Window()
    br = branch.Branch.open("/home/javier/python/bzr.gedit/bzr-gedit")
    options = {"--remember":["Remember location",'B'],
               "--use-existing-dir": ["Use existing dir","B"],
               "--overwrite":["Overwrite","B"],
               "--reference": ['Reference',"T"],
               '--revision':['Revision','R',br]}
    
    ob = OptionsBox(options)
    bt = gtk.Button("ok")
    vbox = gtk.VBox()
    wnd.add(vbox)
    vbox.pack_start(ob)
    vbox.pack_start(bt)
    bt.connect("clicked",clik)
    wnd.show_all()
    #tree.set_path()
    gtk.main()


